package tdspp.Map;

import Prism.core.*;
import tdspp.util.*;
import java.util.*;
import java.awt.Image;

/**
 * The map component. Makes use of class MapImp.
 *
 * @see		MapImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 * 
*/

public class Map extends AbstractImplementation {
	/**
	 * The MapImp object
	 *
	 */
	MapImp mapIO;
	
	/**
	 * Whether a GetMap request is pending
	 */
	boolean pendingGetMap;
	
	/**
	 * Default constructor
	 */
	public Map() {
		this("Map");
	}

	/**
	 * Constructor
	 * 
	 * @param r	Request
	 *
	 */
	public Map(String name) {
		
		pendingGetMap = false;
		if(mapIO == null) {
			mapIO = new MapImp(this);
		}
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public synchronized void handle (Event r) {
		if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAP)) {
			// getmap request
			int submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID)).intValue();
			//System.out.println("TEMP:Map:  Got request for " + submapId);
			Region[] regions = new Region [100];
			Region[] clonedRegions = new Region [100];
			regions = mapIO.getRegions(submapId);
			clonedRegions = (Region[]) regions.clone();
			//System.out.println("\nregions[0].armyRes.resources = " + regions[0].armyRes.resources);
			//System.out.println("\nclonedRegions[24].armyRes.resources = " + clonedRegions[24].armyRes.resources);
			if(regions == null) {
				pendingGetMap = true;
				// pass the request to the Repository
				Event dataReq = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETDATA);
				dataReq.eventType=PrismConstants.REQUEST;
				dataReq.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATA_SUBMAPID,new Integer(submapId));
				send(dataReq);
			}
			else {
				//System.out.println("sending response to GetMap");
				Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAPRESP);
				res.eventType=PrismConstants.REPLY;
				res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,clonedRegions);
				TDSImage tdsImage = mapIO.getTDSImage (submapId);
				//res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_GETMAPRESP_IMAGENAME ,tdsImage.inputName);
				res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_IMAGE,tdsImage);
				send(res);			
			}
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETATTR)) {
			// getattributes request
			Region reg = new Region ();
			Region result = new Region ();
			reg = (Region) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETATTR_REGION);
			result = mapIO.getAttributes(reg);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETATTRRESP);
			res.eventType=PrismConstants.REPLY;
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETATTRRESP_REGION,result);
			send(res);
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETMAP)) {
			// getmap request
			int submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETMAP_SUBMAPID)).intValue();
			Region[] newRegions = new Region [100];
			newRegions = (Region[]) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETMAP_REGIONS);
			Region[] clonedRegions = new Region [100]; 
			clonedRegions = (Region[]) newRegions.clone();
			//System.out.println("MAP:: receivedSETMAP clonedRegions[24].armyRes.resources = "+clonedRegions[24].armyRes.resources);
			
			//System.out.println(""+(Region[]) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETMAP_REGIONS));
			if(submapId == newRegions[0].submapId)
				mapIO.update(submapId, newRegions);
			else System.out.println("not the same region");
			
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_UPDATEUI );
			res.eventType=PrismConstants.REPLY;
			res.addParameter (tdspp.util .MessageConstants .PARAM_NAME_SETMAP_SUBMAPID , new Integer (submapId));
			res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_SETMAP_REGIONS1 , clonedRegions);
			send(res);
			
			/*
			Notification res = new Notification(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAPRESP);
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,newRegions);
			TDSImage tdsImage = mapIO.getTDSImage (submapId);
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_IMAGE,tdsImage);
			send(res);
			*/
			// let the request go to Repository as well
			//return false;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTR)) {
			// setattributes request
			Region reg = new Region ();
			reg = (Region) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETATTR_REGION);
			mapIO.setAttributes(reg);
			//System.out.print("updated cell "+reg.x+" "+reg.y);
			//System.out.println(" value resources is "+reg.armyRes.resources);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTRRESP);
			res.eventType=PrismConstants.REPLY;
			send(res);
			// let the request go to Repository as well
			//return false;
		} else if (r.name.equals(tdspp.util .MessageConstants .MESSAGE_NAME_SAVESAKBSTATE )){
			Event rr = new Event (tdspp.util.MessageConstants .MESSAGE_NAME_SAVESAKB_MAPSTATE );
			rr.eventType=PrismConstants.REQUEST;
			rr.addParameter (tdspp.util .MessageConstants.PARAM_NAME_PASSREGIONS , mapIO.getAllRegions ());
			String[] factors = (String[]) r.getParameter (tdspp.util.MessageConstants .PARAM_NAME_ALLFACTORS );
			Integer[] values = (Integer[]) r.getParameter (tdspp.util .MessageConstants .PARAM_NAME_ALLVALUES );
			rr.addParameter (tdspp.util.MessageConstants.PARAM_NAME_ALLFACTORS , factors);
			rr.addParameter (tdspp.util .MessageConstants .PARAM_NAME_ALLVALUES , values);
			send(rr);
			//return true;
		} else if (r.name.equals(tdspp.util.MessageConstants .MESSAGE_NAME_GETMAPREGIONS )){
			int submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID)).intValue();
			//System.out.println("submapId = "+submapId);
			Region[] regions = new Region [100];
			Region[] clonedRegions = new Region [100];
			regions = mapIO.getRegions(submapId);
			clonedRegions = (Region[]) regions.clone();
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAPREGIONSRESP);
			res.eventType=PrismConstants.REPLY;
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,clonedRegions);
			res.addParameter (tdspp.util .MessageConstants .PARAM_NAME_GETMAP_SUBMAPID , new Integer (submapId));
			send(res);
			//return true;
		} else if (r.name.equals(tdspp.util .MessageConstants .MESSAGE_NAME_REFRESHREGIONS )) {
			int submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID)).intValue();
			Region[] regions = new Region [100];
			Region[] clonedRegions = new Region [100];
			regions = mapIO.getRegions(submapId);
			clonedRegions = (Region[]) regions.clone();
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_UPDATEUI );
			res.eventType=PrismConstants.REPLY;
			res.addParameter (tdspp.util .MessageConstants .PARAM_NAME_SETMAP_SUBMAPID , new Integer (submapId));
			res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_SETMAP_REGIONS1 , clonedRegions);
			send(res);
			//return true;
		} 
		//return false;
		else handleN(r);
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public synchronized void handleN (Event n) {
		if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTRRESP)) {
			// setattributes reponse
			//return true;
		//} else if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETMAPRESP)) {
			// setmap reponse
			//return true;
		} else if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHERRESP)) {
			// getweather reponse
			//return true;
		} else if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHLISTRESP)) {
			// getweatherlist reponse
			if(pendingGetMap == true) {
				pendingGetMap = false;
				Region[] regions = new Region [100];
				regions = (Region[]) n.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHLISTRESP_REGIONS);
				Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAPRESP);
				res.eventType=PrismConstants.REPLY;
				res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,regions);
				TDSImage tdsImage = mapIO.getTDSImage (regions[0].submapId);
				//res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_GETMAPRESP_IMAGENAME ,tdsImage.inputName);
				res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_IMAGE,tdsImage);
				send(res);
			}
			//return true;
		} else if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETDATARESP)) {
			// get data response from the repository
			Region[] regions = new Region [100];
			regions = (Region[]) n.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATARESP_REGIONS);
			int submapId = ((Integer) n.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATARESP_SUBMAPID)).intValue();
			mapIO.update(submapId,regions);
			TDSImage tdsImage = (TDSImage) n.getParameter (tdspp.util .MessageConstants .PARAM_NAME_GETDATARESP_IMAGE );
			tdsImage.img = java.awt.Toolkit.getDefaultToolkit().getImage(tdsImage.outputName);
			mapIO.setTDSImage(submapId,tdsImage);
			// get the weather info
			Event weatherReq = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHLIST);
			weatherReq.eventType=PrismConstants.REQUEST;
			weatherReq.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHLIST_REGIONS,regions);
			send(weatherReq);
			//return true;
		} else if (n.name.equals(tdspp.util .MessageConstants .MESSAGE_NAME_GETALLDATARESP )){
			mapIO.setAllRegions ((Object[])n.getParameter (tdspp.util.MessageConstants.PARAM_NAME_GETALLDATARESP_REGIONS));
			mapIO.setAllTDSImages ((TDSImage[])n.getParameter (tdspp.util.MessageConstants.PARAM_NAME_GETALLDATARESP_IMAGE));
			//return true;
		}
		//return false;
	}
}
