
package tdspp.RenderingAgent;

import java.util.*;
import tdspp.util.*;
import Prism.core.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 * This is the main rendering agent that shows the interface to the user.
 *
 * This being the main GUI, virtually all exposed actions go through this agent. Thus it just
 * acts as a gateway for all that functionality.
 *
 * <table border=0>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Required, Active, User driven </td>
 * </table>
 *
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 *
*/

public class RenderingAgentImp extends GenericImp {
	/**
	 * The main frame
	 */
	public TDSMainFrame frm;

	/**
	 * Id of the submap being displayed
	 */
	public int submapId;

	/**
	 * submaps I am allowed to change
	 */
	public int mySubmaps[];

	/**
	 * The Regions data
	 */
	Region[] regions;

	/**
	 * The Image data
	 */
	TDSImage tdsImage;

	/**
	 * GUI mode
	 */
	public short modeGUI;

	/**
	 * Flag if previous operation move
	 */
	public boolean moveFlag;

	/**
	 * Incomplete message (request);
	 */
	Prism.core.Event incompleteReq;


	/**
	 * Constructor
	 *
	*/
	public RenderingAgentImp(AbstractImplementation parent) {
		super(parent);
		frm = null;
		mySubmaps = null;
		regions = new Region [100];
		incompleteReq = null;
		moveFlag = false;
		modeGUI = tdspp.util.Constants.RENDERINGAGENT_MODE_GETTING;
	}

	/**
	 * set submap to view
	 */
	public void setSubmap(int submap) {
		submapId = submap;
		if(frm != null) frm.checkSubmapMine();
	}

	/**
	 * set region data for the map
	 */
	public void setRegions(Region[] newRegions) {
		this.regions = newRegions;
		//System.out.println("setRegions: regions[24].armyRes.resources = "+regions[24].armyRes.resources);
	}

	/**
	 * set image data for the map
	 */
	public void setTDSImage(TDSImage img) {
		//this.tdsImage = (TDSImage)img.clone();
		this.tdsImage = new TDSImage ();
		this.tdsImage .inputName = img.inputName;
		this.tdsImage .outputName = img.outputName;
		this.tdsImage .img = img.img;
	}

	/**
	 * refresh view
	 */
	public void refresh() {
		frm.tp.invalidate();
		frm.tp.repaint();
		for(int i = 0; i < tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for(int j = 0; j < tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; j++) {
				Region reg = new Region();
				reg = regions[(i * tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP) + j];
				frm.tp.drawIcons(reg,null,true);
			}
	}

	/**
	 * Show deployment advice
	 */
	public void showDeployAdvice(Vector regionVec, Vector levelVec) {
		this.frm.tp.showDeployAdvice(regionVec, levelVec);
		this.modeGUI = tdspp.util.Constants.RENDERINGAGENT_MODE_DEPLOYADV;
		this.frm.bp.updateGUIMode();
	}

	/**
	 * Show simulation UI
	 */
	public void showSimulation(boolean startSim) {
		if(startSim) {
			if(this.modeGUI == tdspp.util.Constants.RENDERINGAGENT_MODE_STARTSIM) {
				this.modeGUI = tdspp.util.Constants.RENDERINGAGENT_MODE_SIMULATION;
				this.frm.bp.updateGUIMode();
			}
		} else {
			this.modeGUI = tdspp.util.Constants.RENDERINGAGENT_MODE_SIMACCEPT;
			this.frm.bp.updateGUIMode();
		}
	}

	/**
	 * Shows the map of a particular area by contacting the Map component. It gets the infomation from
	 * the Map and tries to render it onto the screen.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	MapImp#getMap
	 * @return	void
	 *
	*/
	public void showMap(Region r) {
	}

	/**
	 * Shows the map with weather information of a particular area by contacting the Map
	 * and the Weather components. It gets the infomation from these two components.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	WeatherAnalysisImp#getWeather
	 * @return	void
	 *
	*/
	public void showWeather(Region r) {
	}

	/**
	 * Shows the strategy by contacting the StrategyAnalysisKB component.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	void
	 *
	*/
	public void showStrategy(Region r) {
	}

	/**
	 * Gives the information about the available resources.
	 * Called by: None
	 *
	 * @see 	ResourceManagerImp#availableResources
	 * @return	void
	 *
	*/
	public void availableTroops() {
	}

	/**
	 * Analyzes a particular situation or strategy.
	 * Called by: None
	 *
	 * @return	void
	 *
	*/
	public void analyzeSituation() {
	}

	/**
	 * Create and show the GUI
	 * Called by: None
	 *
	 * @return	void
	 *
	*/
	public void show() {
		if(frm != null) return;
		frm = new TDSMainFrame("TDS++",this);
		if(frm != null) frm.checkSubmapMine();
		Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAP);
		req.eventType=PrismConstants.REQUEST;
		req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID,new Integer(submapId));
		parent.send(req);
	}

	/**
	 * Save the request for move resources
	 * Called by: None
	 *
	 * @return	void
	 *
	*/

	public void saveRequest(Prism.core.Event r) {
		this.incompleteReq = r;
	}
	/**
	 * Set my submaps
	 */
	public void setMySubmaps(int submaps[]) {
		mySubmaps = new int[submaps.length];
		for(int i = 0; i < submaps.length; i++)
			mySubmaps[i] = submaps[i];
	}
}



