package tdspp.Repository;

import Prism.core.*;
import tdspp.util.*;
import java.awt.Image;

/**
 * The repository component. Makes use of class RepositoryImp.
 *
 * @see		RepositoryImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 * 
*/

public class Repository extends AbstractImplementation {
	/**
	 * The RepositoryImp object
	 *
	 */
	RepositoryImp repositoryIO;

	/**
	 * Default constructor
	 */
	public Repository() {
		this("Repository");
	}

	/**
	 * Constructor
	 * 
	 */
	public Repository(String name) {
		
		if(repositoryIO == null) {
			repositoryIO = new RepositoryImp(this);
		}
	}

	/**
	 * set filename
	 */
	public void setFilename(String filename) {
		Constants.REPOSITORY_FILENAME = filename;
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle (Event r) {
		if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETDATA)) {
			// getdata request
			int submapId = ((Integer)r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATA_SUBMAPID)).intValue();
			Region[] regions = repositoryIO.getRegions(submapId);
			TDSImage tdsImage = repositoryIO.getTDSImage(submapId);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETDATARESP);
			res.eventType=PrismConstants.REPLY;
			System.out.println("repos. sent regions");
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATARESP_SUBMAPID,new Integer(submapId));
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATARESP_REGIONS,regions);
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETDATARESP_IMAGE,tdsImage);
			send(res);
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETALLDATA)) {
			// getalldata request
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETALLDATARESP);
			res.eventType=PrismConstants.REPLY;
			System.out.println("repos. sent regions");
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETALLDATARESP_REGIONS,repositoryIO.getAllRegions());
			res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_GETALLDATARESP_IMAGE , repositoryIO.getAllTDSImages());
			send(res);
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETMAP)) {
			// getmap request
			int submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETMAP_SUBMAPID)).intValue();
			Region[] newRegions = (Region[]) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETMAP_REGIONS);
			repositoryIO.update(submapId, newRegions);
			//Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_SETMAPRESP);
			//res.eventType=PrismConstants.REPLY;
			//send(res);
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTR)) {
			// setattributes request
			Region reg = (Region) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_SETATTR_REGION);
			repositoryIO.setAttributes(reg);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTRRESP);
			res.eventType=PrismConstants.REPLY;
			send(res);
			//return true;
		} else if (r.name.equals(tdspp.util.MessageConstants .MESSAGE_NAME_SAVESAKB_MAPSTATE )){
			//System.out.println("SavesakbMAPState RECEIVED");
			repositoryIO.setFactors ((String[])r.getParameter (tdspp.util .MessageConstants .PARAM_NAME_ALLFACTORS ));
			repositoryIO.setValues ((Integer[]) r.getParameter (tdspp.util .MessageConstants .PARAM_NAME_ALLVALUES ));
			repositoryIO.setAllRegions ((Object[])r.getParameter (tdspp.util .MessageConstants .PARAM_NAME_PASSREGIONS ));
			if (repositoryIO.getAllRegions () == null) System.out.println("null");
			if (repositoryIO.getFactors () == null) System.out.println("null");
			if (repositoryIO.getValues () == null) System.out.println("null");
			System.out.println("XXXXXXXXXXXXX Repository.start() : Writing file .. XXXXXXXXXXXXXX");
			repositoryIO.writeFile(Constants.REPOSITORY_FILENAME);
			//return true;
		}
		//return false;
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */


	/**
	 * Start functioning
	 */
	public void start() {
		System.out.println("Repository.start() : Reading file ["+ Constants.REPOSITORY_FILENAME + ".. ");
		repositoryIO.readFile(Constants.REPOSITORY_FILENAME);
		
		//added by Vlada
		Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETALLDATARESP);
		res.eventType=PrismConstants.REPLY;
		res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETALLDATARESP_REGIONS,repositoryIO.getAllRegions());
		res.addParameter (tdspp.util .MessageConstants.PARAM_NAME_GETALLDATARESP_IMAGE , repositoryIO.getAllTDSImages());
		send(res);
			
		Event n = new Event (tdspp.util .MessageConstants .MESSAGE_NAME_GETINITIALRULESRESP );
		n.eventType=PrismConstants.REPLY;
		n.addParameter (tdspp.util .MessageConstants.PARAM_NAME_NUMBERVALUE , new Integer (repositoryIO.factNum ));//length of 2 arrays
		n.addParameter (tdspp.util .MessageConstants.PARAM_NAME_ALLFACTORS , repositoryIO.getFactors ());
		n.addParameter (tdspp.util .MessageConstants .PARAM_NAME_ALLVALUES , repositoryIO.getValues ());
		send(n);
		
		//System.out.println("XXXXXXXXXXXXX Repository.start() : Writing file .. XXXXXXXXXXXXXX");
		//repositoryIO.writeFile(Constants.REPOSITORY_FILENAME);
		
	}
}
