package tdspp.ResourceManager;


import Prism.core.*;
import tdspp.util.*;


/**
 * Manages the resources availabel to the system.
 *
 * Keeps a list of the region handles and the resources used by them. In addition
 * it keeps track of the total resources in the system.
 *
 * <table border=0>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Variant, Passive </td>
 * </table>
 *
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 *
*/

public class ResourceManagerImp extends GenericImp {

/**
 *		Queue of Names of Resources to be managed in a given region
 */
 	private String resourceNames[][];

/**
 *		Operation indicator for each region
 */
 	private int operationFlag[];	//operationFlag null ->  get
									//				1 ->  add
									//				-1 ->  delete
									//				2 -> move
									//				3 -> supply Resources
									//				4 -> deplete Resources
	private static int flagPeriodic;

/**
 *		Queue of Regions to which resources have to be added/deleted
 */
	private Region reg[];			//common queue
	private Region reg2[];			//queue for destination regions
	private Region regPeriodic;		//queue for regions to be supplied or deplted resources


/**
 *		Queue parameters
 */
	private int queueLength;
	private int queueTop;

	private static int QUEUESIZE = 5;


/**
 *		Constructor
 *		@param parent	Component
 */

	public ResourceManagerImp(AbstractImplementation parent) {
		super(parent);
		this.reg = new Region [QUEUESIZE];
		this.reg2 = new Region [QUEUESIZE];
		this.resourceNames = new String [QUEUESIZE][10];
		this.operationFlag = new int [QUEUESIZE];
		this.queueLength = -1;
		this.queueTop = 0;
	}


	/*
	 *	Save latest operation request on Region
	 *	@param resourceNames String []
	 *	@param opFlag int
	 *	@param reg Region
	 */
	public synchronized void setAttributes(String resourceNames[], int opFlag, Region reg) {
		addQ();
		this.reg[queueLength] = reg;
		this.reg2[queueLength] = null;
		//this.resourceNames = new String[resourceNames.length];

		for(int i = 0; i < resourceNames.length; i++) {
			if(resourceNames[i] != null)
				this.resourceNames[queueLength][i] = resourceNames[i];
			else
				this.resourceNames[queueLength][i] = null;
		}

		this.operationFlag[queueLength] = opFlag;
	}

	/*
	 *	Save latest move operation request on Region
	 *	@param resourceNames String []
	 *	@param opFlag int
	 *	@param reg Region
	 *	@param reg2 Region
	 */
	public synchronized void setAttributes(String resourceNames[], int opFlag, Region reg, Region reg2) {
		addQ();
		this.reg[queueLength] = reg;
		this.reg2[queueLength] = reg2;

		//this.resourceNames = new String[resourceNames.length];
		for(int i = 0; i < resourceNames.length; i++) {
			if(resourceNames[i] != null)
				this.resourceNames[queueLength][i] = resourceNames[i];
			else
				this.resourceNames[queueLength][i] = null;
		}
		this.operationFlag[queueLength] = opFlag;
		//printQ();
	}



	/*
	 *	Save latest supply deplete operation request on Region
	 *	@param opFlag int
	 *	@param reg Region
	 *  @return Region
	 */
	public synchronized void setAttributes(int opFlag, Region reg) {
		this.regPeriodic = reg;
		this.flagPeriodic = opFlag;
	}




	/*
	 *	Sets the resource bits if the resource is added
	 *	@param reg Region
	 */

	public synchronized Region addResources(Region reg){
		//System.out.println("Add Resources");


		for(int i = 0; i < resourceNames[queueTop].length; i++) {
			if(this.resourceNames[queueTop][i] != null) {
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[0])){
					reg.armyRes.setInfantry();
					reg.armyRes.setStatePopulation((float)1.0);
					reg.armyRes.setStateFood((float)1.0);
					reg.armyRes.setStateAmmunition((float)1.0);
				}

				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[1])){
					reg.armyRes.setTank();
				}
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[2])){
					reg.armyRes.setMineField();
				}

				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[3])){
					reg.armyRes.setEnemyInfantry();
					reg.armyRes.setStatePopulation((float)1.0);
					reg.armyRes.setStateFood((float)1.0);
					reg.armyRes.setStateAmmunition((float)1.0);
				}

				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[4])){
					reg.armyRes.setEnemyTank();
				}
			}
		}
		removeQ();
		return reg;
	}

	/*
	 *	Resets the resource bits if the resource is deleted
	 *	@param reg Region
	 *  @return Region
	 */

	public synchronized Region deleteResources(Region reg){
/*		System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++");
		//System.out.println("TEMP: ResMgrImp: Delete Resources");
		//System.out.println("TEMP: ResMgrImp: " + reg);
*/

		for(int i = 0; i < resourceNames[queueTop].length; i++) {
			//System.out.println("TEMP: ResMgrImp: " + this.resourceNames[queueTop][i]);
			if(this.resourceNames[queueTop][i] != null) {
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[0])){
					reg.armyRes.removeInfantry();
					reg.armyRes.setStatePopulation((float)0.0);
					reg.armyRes.setStateFood((float)0.0);
					reg.armyRes.setStateAmmunition((float)0.0);
				}
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[1])){
					reg.armyRes.removeTank();
				}
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[2])){
					reg.armyRes.removeMineField();
				}
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[3])){
					reg.armyRes.removeEnemyInfantry();
					reg.armyRes.setStatePopulation((float)0.0);
					reg.armyRes.setStateFood((float)0.0);
					reg.armyRes.setStateAmmunition((float)0.0);

				}
				if(this.resourceNames[queueTop][i].equalsIgnoreCase(tdspp.util.Constants.ARMYRESOURCESNAMES[4])){
					reg.armyRes.removeEnemyTank();
				}


			}
		}
/*		System.out.println("TEMP: ResMgrImp: " + reg);
		System.out.println("--------------------------------------------------");
		removeQ();
*/		return reg;
	}

	/*
	 *	Decides which update operation is to be carried out
	 *	@param reg Region
	 *	@return Region
	 */

	public synchronized Region updateResources(Region reg) {
		//System.out.println(queueTop);
		//printQ();
		//System.out.println("updating...." + queueTop + queueLength);
		//System.out.println("regns" + this.reg[queueTop] + reg);
		if(this.reg[queueTop].x == reg.x  && this.reg[queueTop].y == reg.y) {
			//System.out.println("reg 1....");
			if(operationFlag[queueTop] == 1){
				reg = addResources(reg);
			}
			else {
				if(operationFlag[queueTop] == -1 || operationFlag[queueTop] == 2) {
					/*System.out.println("=====xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx=====");
					System.out.println("========================================");
					System.out.println("Deleteing from ..... " + reg);*/
					reg = deleteResources(reg);
					/*System.out.println("");
					System.out.println("Nsq vweaion..... " + reg);
					System.out.println("========================================");
					*/
				}
			}
		}
		else if(this.reg2[queueTop].x == reg.x  && this.reg2[queueTop].y == reg.y) {
			if(operationFlag[queueTop] == 2) {
				/*System.out.println("========================================");
				System.out.println("adding..... " + reg);*/
				reg = addResources(reg);
				/*System.out.println("");
				System.out.println("Nsq vweaion..... " + reg);
				System.out.println("========================================");
				*/
			}
			removeQ();
		}
		return reg;
	}



	/*
	 *	Updates the state fields of the regions in a given row to increase the resources
	 *	@param regArray Region[]
	 *	@return Region[]
	 */

	public Region[] supplyResources(Region regArray[]) {
		//System.out.println("Supplying ..... " + regPeriodic.x);
		int XLENGTH = tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP;

		for(int i = 0; i < tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++) {
			if(regArray[(regPeriodic.x * XLENGTH) + i].armyRes.isInfantry()) {

				float temp;

				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateAmmunition() * (float)(Math.random());
				temp += regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateAmmunition();
				if(temp > (float)1.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateAmmunition((float)1.0);
				else if(temp == (float)0.0)
					regArray[(regPeriodic.x * XLENGTH)  + i].armyRes.setStateAmmunition((float)0.3);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateAmmunition(temp);

				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateFood() * (float)(Math.random());
				temp += regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateFood();
				if(temp > (float)1.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateFood((float)1.0);
				else if(temp > (float)0.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateFood((float)0.3);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateFood(temp);

				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStatePopulation() * (float)(Math.random());
				temp += regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStatePopulation();
				if(temp > (float)1.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStatePopulation((float)1.0);
				else if(temp == (float)0.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStatePopulation((float)0.3);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStatePopulation(temp);

		/*		System.out.println("Region map id " + regArray[regPeriodic.x + i].submapId + " x " +
					regArray[(regPeriodic.x * XLENGTH) + i].x + " y " +
						regArray[(regPeriodic.x * XLENGTH) + i].y + " total " +
							regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getTotalState());
		*/
			}
		}
		return regArray;
	}


	/*
	 *	Updates the state fields of the regions in a given row to decrease the resources
	 *	@param regArray Region[]
	 *	@return Region[]
	 */

	public Region[] depleteResources(Region regArray[]) {
		int XLENGTH = tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP;
		for(int i = 0; i < tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++) {

			if(regArray[(regPeriodic.x * XLENGTH) + i].armyRes.isInfantry()) {

				float temp;
				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateAmmunition() * (float)(Math.random());
				temp -= regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateAmmunition();
				if(temp < (float)0.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateAmmunition((float)0.0);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateAmmunition(temp);

				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateFood() * (float)(Math.random());
				temp -= regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStateFood();
				if(temp < (float)0.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateFood((float)0.0);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStateFood(temp);

				temp = regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStatePopulation() * (float)(Math.random());
				temp -= regArray[(regPeriodic.x * XLENGTH) + i].armyRes.getStatePopulation();
				if(temp < (float)0.0)
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStatePopulation((float)0.0);
				else
					regArray[(regPeriodic.x * XLENGTH) + i].armyRes.setStatePopulation(temp);
			}
		}
		return regArray;
	}

	/*
	 *	Returns operation flag of oldest outstanding region
	 *	@param void
	 *	@return int
	 */


	public synchronized int getOpFlag() {
			//System.out.println("queuetop" + queueTop);
			return this.operationFlag[queueTop];
	}

	/*
	 *	Returns operation flag of oldest region
	 *	@param void
	 *	@return int
	 */

	public synchronized int getPeriodicFlag() {
			return this.flagPeriodic;
	}


	/*
	 *	Slide Queue  Length
	 *	@param void
	 *	@return void
	 */
	private synchronized void addQ() {
		//System.out.println("Addin to queue >>> " + queueLength + " " + queueTop);
		//if(queueTop >= 0 && queueLength != QUEUESIZE) {
			queueLength = (queueLength + 1) % QUEUESIZE;
		//	System.out.println("Added to queue >>> " + queueLength + " " + queueTop);

		//}
	}



	/*
	 *	Slide Queue  Top
	 *	@param void
	 *	@return void
	 */
	private synchronized void removeQ() {
		//System.out.println("RemovIN queue >>> " + queueLength + " " + queueTop);
		//if(queueTop != queueLength)
		//{
			queueTop = (queueTop + 1) % QUEUESIZE;
			if(queueTop > queueLength) {
				queueTop = 0;
				queueLength = -1;
			}
		//System.out.println("Removed queue >>> " + queueLength + " " + queueTop);
		//}

	}

	private synchronized void printQ() {
		//System.out.println("Queue >>>>>>>>>>> ");
		for(int i = queueTop; i <= queueLength; i++) {
			System.out.println(" i " + i + "opFlag " + operationFlag[i]);
		}
	}

}
