package tdspp.SimulationAgent;

import java.util.*;
import tdspp.util.*;
import Prism.core.*;

/**
 * This component provides the strategies to the user.
 *
 * This is the interface of the system to the user who wishes to simulate some conditions.
 * This can be implemented as an independent GUI or a module feding into the RenderingAgent.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Optional, Variant, Active </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 * 
*/

public class SimulationAgentImp extends GenericImp {
	Region[] regions;
	Region [] prevRegions;
	Region[][] territory;
	float friends, enemies;
	Hashtable localRules;
	/**
	 * Constructor
	 * 
	*/
	public SimulationAgentImp() {
		territory = new Region[tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP][tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 regions = new Region [tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP*tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 prevRegions = new Region [tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP*tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 localRules = new Hashtable();
		 
	}
	
	public void setRegions(Region[] regions){
		this.regions = (Region[]) regions.clone();
		for (int i=0; i<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (int j=0; j<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				territory[i][j]= regions[10*i+j];
	
			}
	}
	
	public void saveRegions(Region[] regions){
		this.prevRegions = (Region[]) regions.clone();
	}
	
	public Region[] getPrevRegions(){
		return prevRegions;	
	}
	
	
	public void setLocalRules(Hashtable localRules){
		this.localRules = localRules;
		//System.out.println("Size of localRules is "+localRules.size());
	}
	
	
	public Region[] getRegions(){
		for (int i=0; i<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (int j=0; j<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; j++)
				regions[10*i+j] = territory[i][j];
		return regions;
	}
	
	private void weakRegion(int i, int j){
		//float coef = (float) 0.85;
		float oldAmmunition = territory[i][j].armyRes.getStateAmmunition ();
		territory[i][j].armyRes .setStateAmmunition ((float)((Constants.COEF-(float)0.01) *oldAmmunition));
		float oldFood = territory[i][j].armyRes.getStateFood ();
		territory[i][j].armyRes.setStateFood ((float)(Constants.COEF*oldFood));
		float oldPopulation = territory[i][j].armyRes.getStatePopulation();
		territory[i][j].armyRes.setStatePopulation ((float)((Constants.COEF+(float)0.01)*oldPopulation));
		if (territory[i][j].armyRes.getStateAmmunition () 
			 +territory[i][j].armyRes.getStateFood () 
			 +territory[i][j].armyRes.getStatePopulation() <= (float)0.9){
			territory[i][j].armyRes.setEmpty ();
			territory[i][j].armyRes.setStateAmmunition ((float)0.0);
			territory[i][j].armyRes.setStateFood ((float)0.0);
			territory[i][j].armyRes.setStatePopulation ((float)0.0);
			regions[10*i+j].armyRes .setEmpty ();
			regions[10*i+j].armyRes.setStateAmmunition ((float)0.0);
			regions[10*i+j].armyRes.setStateFood ((float)0.0);
			regions[10*i+j].armyRes.setStatePopulation ((float)0.0);
			
		}
	}
	
	public void simulateFight(){
		int i, j;
		float wellness, coefEnemies, coefFriends;
		friends=(float)0.0;
		enemies=(float)0.0;
		for (i=0; i<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				
				wellness = territory[i][j].armyRes.getStateAmmunition () + territory[i][j].armyRes.getStateFood () + territory[i][j].armyRes.getStatePopulation ();
				if (territory[i][j].armyRes .isInfantry ()){
					if (territory[i][j].terrainDesc .getType()==tdspp.util.Constants.TERRAINTYPE_MOUNTAIN )
						friends+=1.2*wellness;
					else friends+=1*wellness;
				}
				if (territory[i][j].armyRes.isTank ()){
					if (territory[i][j].terrainDesc .getType()==tdspp.util.Constants.TERRAINTYPE_MOUNTAIN )
						friends+=2.4*wellness;
					else friends+=2*wellness;
				}
				if (territory[i][j].armyRes.isEnemyInfantry ()){
					if (territory[i][j].terrainDesc .getType()==tdspp.util.Constants.TERRAINTYPE_MOUNTAIN )
						enemies+=1.2*wellness;
					else enemies+=1*wellness;
				}
				if (territory[i][j].armyRes.isEnemyTank ()){
					if (territory[i][j].terrainDesc .getType()==tdspp.util.Constants.TERRAINTYPE_MOUNTAIN )
						enemies+=2.4*wellness;
					else enemies+=2*wellness;
				}
			}
		for (i=0; i<tdspp.util .Constants .REPOSITORY_CELLS_PER_SUBMAP ; i++)
			for (j=0; j<tdspp.util .Constants .REPOSITORY_CELLS_PER_SUBMAP; j++){
				if (territory[i][j].armyRes .isInfantry () || territory[i][j].armyRes.isTank ()){
					if (friends >= enemies) {
						if (enemies > 0){
							coefFriends = (float)0.5 - (friends - enemies)/(2*enemies);
							if (Math.random()<coefFriends){
								weakRegion(i,j);
							}
						}
					}
					else {//more enemies
						if (friends > 0){
							coefEnemies = (float)0.5 + (enemies - friends)/(2*friends);
							if (Math.random()<coefEnemies){
								weakRegion(i,j);
							}
						}
					}
				}
				else if (territory[i][j].armyRes.isEnemyInfantry () || territory[i][j].armyRes.isEnemyTank ()){
					if (friends > enemies) {
						if (enemies > 0){
							coefFriends = (float)0.5 + (friends - enemies)/(2*enemies);
							if (Math.random()<coefFriends){
								weakRegion(i,j);
							}
						}
					}
					else {//more enemies
						if (friends > 0){
							coefEnemies = (float)0.5 - (enemies - friends)/(2*friends);
							if (Math.random()<coefEnemies){
								weakRegion(i,j);
							}
						}
					}
				}
			}		
		
	}
	
	
}
	