package tdspp.StrategyAnalyzerAgent;
import java.util .*;
import tdspp .util .*;

/**
 * Provides analysis of given strategies to the user.
 *
 * It is implemented as an interface to the StrategyAnalysisKB. Some rules are fed into the KB and then
 * this component makes use of that information to analyze the given strategy.
 * can just be the entensions to this class. This class contains the sample
 * javadoc comments.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Optional, Active </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 15 Feb 2002
 * 
*/

public class StrategyAnalyzerAgentImp extends GenericImp {
	Region[] regions;
	Region[][] territory;
	boolean valid = true;
	Vector invalidRegion, invalidLevel;
	Hashtable localRules;
	int[][] contrLevel;//contribution level for every region
	
	public StrategyAnalyzerAgentImp(){
		 territory = new Region[Constants.REPOSITORY_CELLS_PER_SUBMAP][Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 regions = new Region [Constants.REPOSITORY_CELLS_PER_SUBMAP*Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 contrLevel = new int [Constants.REPOSITORY_CELLS_PER_SUBMAP][Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 localRules = new Hashtable();
		 invalidRegion = new Vector ();
		 invalidLevel = new Vector ();
	
	}
	
	public void setLocalRules(Hashtable localRules){
		this.localRules = localRules;
		//System.out.println("Size of localRules is "+localRules.size());
	}
	
	public void setRegions(Region[] regions){
		this.regions = (Region[]) regions.clone();
		for (int i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (int j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				territory[i][j]= regions[10*i+j];
				contrLevel[i][j] = 50;	
			}
	}
	
	public boolean isValid(){
		return this.valid ;
	}
	
	public Vector getInvalidRegion(){
		return invalidRegion;
	}
	
	public Vector getInvalidLevel(){
		return invalidLevel;
	}
	
	private void checkArmy (int x, int y, int i, int j) {
		if (x>=0)
			if (x<Constants.REPOSITORY_CELLS_PER_SUBMAP)
				if (y>=0)
					if (y<Constants.REPOSITORY_CELLS_PER_SUBMAP){
						if (territory[x][y].armyRes.isMineField ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_MINEFIELD))
								contrLevel[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_MINEFIELD)).intValue();
							valid = false; 
						}
						else if (territory[x][y].armyRes.isEnemyInfantry ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_ENEMYINFT )){
								contrLevel[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_ENEMYINFT)).intValue();
								//System.out.println("contrLevel "+i+" "+j+" = "+contrLevel[i][j]);
							}
							valid = false;
						}
						else if (territory[x][y].armyRes.isEnemyTank ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_ENEMYTANK ))
								contrLevel[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_ENEMYTANK)).intValue();
							
							valid = false;
						}
					}
		
	}
	
	private void checkTerrain (int x, int y){
		if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_SWAMP) {
			if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_SWAMP ))
				contrLevel[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_SWAMP)).intValue();
			valid = false; 
		}
		else if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_RIVER ){
			if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_RIVER ))
				contrLevel[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_RIVER)).intValue();
			valid = false; 
		}
	}
	
	public void analyzeStrategy(){
		
		int i, j;
		valid=true;
		
		for (i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				if (territory[i][j].armyRes .isInfantry () || territory[i][j].armyRes.isTank ()){
					//System.out.println (i+" "+j);
					checkTerrain(i,j);
					checkArmy(i-1,j-1,i,j);
					checkArmy(i-1,j,i,j);
					checkArmy(i-1,j+1,i,j);
					checkArmy(i,j-1,i,j);
					checkArmy(i,j+1,i,j);
					checkArmy(i+1,j-1,i,j);
					checkArmy(i+1,j,i,j);
					checkArmy(i+1,j+1,i,j);
					
				}
				
			}	
		for (i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++)
				if (contrLevel[i][j] != 50){
					invalidRegion.addElement(new Integer (Constants.REPOSITORY_CELLS_PER_SUBMAP*i+j));
					invalidLevel.addElement(new Integer (contrLevel[i][j]));
				}
	}
	
	public void analyzeStrategy(Region[][] ter){
	
		//System.out.println ("in");
		territory = (Region[][]) ter.clone();//check this
		analyzeStrategy();		
		
	}
	
}	
	
	/**
	 * Analyze the given strategy
	 * 
	 * @param s	The strategy to be analyzed
	 * @see 	#analyzeStrategyForRegion
	 * @see 	#analyzeStrategyForWeather
	 * @see 	#analyzeStrategyForRegionWeather
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	Results of the analysis - grade on scale of 100
	 * 
	*/
	/*
	public int analyzeStrategy(Strategy s) {
		return analyzeStrategyForRegionWeather(s,null,null);
	}
	*/

	/**
	 * Analyze the given strategy for the given region.
	 * 
	 * @param s	The strategy to be analyzed
	 * @param r	The region handle
	 * @see 	#analyzeStrategy
	 * @see 	#analyzeStrategyForWeather
	 * @see 	#analyzeStrategyForRegionWeather
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	Results of the analysis - grade on scale of 100
	 * 
	*/
	/*
	public int analyzeStrategyForRegion(Strategy s, Region r) {
		return analyzeStrategyForRegionWeather(s,r,null);
	}
	*/

	/**
	 * Analyze the given strategy under given weather conditions for the whole map
	 * 
	 * @param s	The strategy to be analyzed
	 * @param wi	The WeatherInfo object
	 * @see 	#analyzeStrategy
	 * @see 	#analyzeStrategyForRegion
	 * @see 	#analyzeStrategyForRegionWeather
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	Results of the analysis - grade on scale of 100
	 * 
	*/
	/*
	public int analyzeStrategyForWeather(Strategy s, WeatherInfo wi) {
		return analyzeStrategyForRegionWeather(s,null,wi);
	}
	*/

	/**
	 * Analyze the given strategy under given weather conditions for the given region.
	 * 
	 * @param s	The strategy to be analyzed
	 * @param r	The region handle
	 * @param wi	The WeatherInfo object
	 * @see 	#analyzeStrategy
	 * @see 	#analyzeStrategyForRegion
	 * @see 	#analyzeStrategyForWeather
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	Results of the analysis - grade on scale of 100
	 * 
	*/
	/*
	public int analyzeStrategyForRegionWeather(Strategy s, Region r, WeatherInfo wi) {
		// analyze and return scale
	}
	*/

