package tdspp.Weather;

import Prism.core.*;
import tdspp.util.*;
import java.util.*;

/**
 * The weather component. Makes use of class WeatherImp.
 *
 * @see		WeatherImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 * 
*/

public class Weather extends AbstractImplementation {
	/**
	 * The WeatherImp object
	 *
	 */
	WeatherImp weatherIO;

	/**
	 * The no of ticks after which the weather information is updated
	 */
	int updInterval;
	
	/**
	 * The UI makes a copy of the data
	 * So this is the no of ticks after which the UI update is called for.
	 */
	int uiUpdInterval;
	
	/**
	 * The no of ticks
	 */
	int tickCounter;
	
	/**
	 * Default constructor
	 */
	public Weather() {
		this("Weather");
	}

	/**
	 * Constructor
	 * 
	 * @param name Name
	 *
	 */
	public Weather(String name) {
		updInterval = Constants.WEATHER_UPDATE_INTERVAL;
		uiUpdInterval = Constants.WEATHER_UI_UPDATE_INTERVAL;
		tickCounter = 0;
		if(weatherIO == null) {
			weatherIO = new WeatherImp(this);
		}
	}

	/**
	 * Set the update interval
	 * 
	 * @param ticks No of ticks before the update.
	 *
	 */
	public void setUpdInterval(int ticks) {
		updInterval = ticks;
	}

	/**
	 * Set UI the update interval
	 * 
	 * @param ticks No of ticks before the UI update is called for.
	 *
	 */
	public void setUIUpdInterval(int ticks) {
		uiUpdInterval = ticks;
	}

	/**
	 * Set the probabilities and increments
	 * 
	*/
	public void setUpdateParams(double updateProb, 
						double tempProb, double humidProb, double windProb,	double sunlightProb,
						double tempIncr, double humidIncr, double windIncr, double sunlightIncr) {
		if(weatherIO != null)
			weatherIO.setUpdateParams(updateProb, tempProb, humidProb, windProb, sunlightProb, tempIncr, humidIncr, windIncr,sunlightIncr);
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle (Event r) {
		if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHER)) {
			// getweather request
			Region reg = (Region) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHER_REGION);
			WeatherInfo wi = weatherIO.getWeather(reg);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHERRESP);
			res.eventType=PrismConstants.REPLY;
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHERRESP_WEATHERINFO,wi);
			send(res);
			//return true;
		} else if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHLIST)) {
			// getweatherlist request
			Region[] regions = (Region[]) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHLIST_REGIONS);
			weatherIO.populateWeatherInfo(regions);
			Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHLISTRESP);
			res.eventType=PrismConstants.REPLY;
			
			res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHLISTRESP_REGIONS,regions);
			send(res);
			//return true;
		}
		//return false;
		else handleN(r);
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public void handleN (Event n) {
		if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_TICK)) {
			tickCounter++;
			if(tickCounter % updInterval == 0) {
				weatherIO.update();
			}
			if(tickCounter % uiUpdInterval == 0) {
				Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_CALLUIUPDATE);
				res.eventType=PrismConstants.REPLY;
			
				send(res);
			}
			// pass the notification to simulation agent
			////return false;
		}
		//return false;
	}
	
	/**
	 * Start functioning
	 */
	public void start() {
		System.out.println("Weather.start() : Reading file ..");
		weatherIO.readFile(Constants.REPOSITORY_FILENAME);
	}
}
