package tdspp.WeatherAnalyzer;

import Prism.core.*;
import tdspp.util.*;

/**
 * The weather analyzer component. Makes use of class WeatherAnalyzerImp.
 *
 * @see		WeatherAnalyzerImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 * 
*/

public class WeatherAnalyzer extends AbstractImplementation {
	/**
	 * The WeatherAnalyzerImp object
	 *
	 */
	WeatherAnalyzerImp weatherAnalyzerIO;

	/**
	 * Default constructor
	 */
	public WeatherAnalyzer() {
		this("WeatherAnalyzer");
	}

	/**
	 * Constructor
	 * 
	 * @param name Name
	 *
	 */
	public WeatherAnalyzer(String name) {
		if(weatherAnalyzerIO == null) {
			weatherAnalyzerIO = new WeatherAnalyzerImp(this);
		}
	}

	/**
	 * Set the weights and multiplication factor
	 * 
	*/
	public void setParams(int wtTemp, int wtHumid, int wtWind, int wtSun, int mulFact) {
		if(weatherAnalyzerIO != null) 
			weatherAnalyzerIO.setParams(wtTemp,wtHumid,wtWind,wtSun,mulFact);
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle (Event r) {
		if(r.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHERLEVEL)) {
			// getweatherlevel request
			Region reg = (Region) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHERLEVEL_REGION);
			Event req = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHER);
			req.eventType=PrismConstants.REQUEST ;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHER_REGION,reg);
			send(req);
			//return true;
		}
		//return false;
		else handleN(r);
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public void handleN (Event n) {
		if(n.name.equals(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHERRESP)) {
			WeatherInfo wi = (WeatherInfo) n.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHERRESP_WEATHERINFO);
			int level = weatherAnalyzerIO.getWeatherLevel(wi);
			if(level != -1) {
				Event res = new Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETWEATHERLEVELRESP);
				res.eventType=PrismConstants.REPLY ;
			
				res.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETWEATHERLEVELRESP_LEVEL,new Integer(level));
				send(res);
			}
			//return true;
		}
		//return false;
	}
}
