package tdspp.apps;

import Prism.framework.*;
import Prism.conn.basic.*;
import Prism.conn.distribution.bottom.*;
import Prism.conn.distribution.top.*;
import Prism.util.*;

import tdspp.Clock.*;
import tdspp.DeploymentAdvisor.*;
import tdspp.Map.*;
import tdspp.RenderingAgent.*;
import tdspp.Repository.*;
import tdspp.ResourceManager.*;
import tdspp.ResourceMonitor.*;
import tdspp.SAKBUI.*;
import tdspp.SimulationAgent.*;
import tdspp.StrategyAnalysisKB.*;
import tdspp.StrategyAnalyzerAgent.*;
import tdspp.util.*;
import tdspp.Weather.*;
import tdspp.WeatherAnalyzer.*;

/**
 * 
 * This is the sample main class that executes the application as the General.
 * 
 */

public class General
{
	/**
	 * The main method
	 */
	public static void main(String argv[]) {
		FIFODispatch disp = new FIFODispatch(100);//size of the message queue
		Scaffold s = new Scaffold(ScaffoldConstants.DISPATCH | ScaffoldConstants.SCHEDULER);
		RoundRobinScheduler sch = new RoundRobinScheduler(disp, 10);//10 threads
		s.setScheduler(sch);
		s.setDispatch(disp);
		ArchitectureAdapter arch=new ArchitectureAdapter("General");
		arch.setScaffold(s);
		System.out.println("Creating the architecture");

		// creating the components
		// first level
		Clock clk = new Clock("clock");
		clk.setScaffold(s);
		arch.add(clk);
		Connector c1 = new Connector("belowClock");
		c1.setScaffold(s);
		arch.add(c1);
		arch.weld(clk,c1);
		// second level
		Weather wth = new Weather("weatherG");
		wth.setScaffold(s);
		wth.setUpdateParams(0.5, 0.6, 0.3, 0.2, 0.4, 2, 2, 5,3);
		wth.setUIUpdInterval(50);
		wth.setUpdInterval(10);
		arch.add(wth);
//		Forwarder f1 = new Forwarder("FWDer1");
//		f1.setScaffold(s);
//		arch.add(f1);
		Repository rep = new Repository("repos");
		if(argv.length > 1) rep.setFilename(argv[1]);
		rep.setScaffold(s);
		arch.add(rep);
		Connector c2 = new Connector("belowRep");
		c2.setScaffold(s);
		arch.add(c2);
		arch.weld(c1,wth);
//		arch.weld(c1,f1);
		arch.weld(c1,rep);
		arch.weld(wth,c2);
//		arch.weld(f1,c2);
		arch.weld(c1,c2);
		arch.weld(rep,c2);
		// third level
		Map map = new Map("map");
		map.setScaffold(s);
		arch.add(map);
//		Forwarder f2 = new Forwarder("FWDer2");
//		f2.setScaffold(s);
//		arch.add(f2);
		Connector c3 = new Connector("belowMap");
		c3.setScaffold(s);
		arch.add(c3);
		arch.weld(c2,map);
//		arch.weld(c2,f2);
//		arch.weld(f2,c3);
		arch.weld(c2,c3);
		arch.weld(map,c3);
		// fourth level
		WeatherAnalyzer wthan = new WeatherAnalyzer("weatherAnalysis");
		wthan.setScaffold(s);
		arch.add(wthan);
//		Forwarder f3 = new Forwarder("FWDer3");
//		f3.setScaffold(s);
//		arch.add(f3);
		ResourceManager resmgr = new ResourceManager("resMgr");
		resmgr.setScaffold(s);
		arch.add(resmgr);
		StrategyAnalysisKB sakb = new StrategyAnalysisKB("SAKB");
		sakb.setScaffold(s);
		arch.add(sakb);
		Connector c4 = new Connector("belowResMgr");
		c4.setScaffold(s);
		arch.add(c4);
		arch.weld(c2,wthan);
//		arch.weld(c3,f3);
		arch.weld(c3,resmgr);
		arch.weld(c3,sakb);
		arch.weld(wthan,c4);
//		arch.weld(f3,c4);
		arch.weld(c3,c4);
		arch.weld(resmgr,c4);
		arch.weld(sakb,c4);
		// fifth level
		SimulationAgent simag = new SimulationAgent("SimAgt");
		simag.setScaffold(s);
		arch.add(simag);
		ResourceMonitor resmon = new ResourceMonitor("ResMon");
		resmon.setScaffold(s);
		arch.add(resmon);
//		Forwarder f4 = new Forwarder("FWDer4");
//		f4.setScaffold(s);
//		arch.add(f4);
		StrategyAnalyzerAgent saa = new StrategyAnalyzerAgent("SAAgent");
		saa.setScaffold(s);
		arch.add(saa);
		DeploymentAdvisor da = new DeploymentAdvisor("depAdv");
		da.setScaffold(s);
		arch.add(da);
		SAKBUI sakbui = new SAKBUI("sakbUI");
		sakbui.setScaffold(s);
		arch.add(sakbui);
		Connector c5 = new Connector("belowAgents");
		c5.setScaffold(s);
		arch.add(c5);
		arch.weld(c4,simag);
		arch.weld(c4,resmon);
//		arch.weld(c4,f4);
		arch.weld(c4,saa);
		arch.weld(c4,da);
		arch.weld(c4,sakbui);
		arch.weld(simag,c5);
		arch.weld(resmon,c5);
//		arch.weld(f4,c5);
		arch.weld(c4,c5);
		arch.weld(saa,c5);
		arch.weld(da,c5);
		arch.weld(sakbui,c5);
		// sixth level
		RenderingAgent ui = new RenderingAgent("UI");
		int submaps[] = { 0, 1, 4, 5 };
		ui.setMySubmaps(submaps);
		if(argv.length > 0) ui.setSubmap(Integer.parseInt(argv[0]));
		else ui.setSubmap(0);
		ui.setScaffold(s);
		arch.add(ui);
		arch.weld(c5,ui);
		// print architecture
//		System.out.println("Architecture looks like " + arch);
		// start it
		sch.start();
		arch.start();
	}
	
}