package tdspp.apps;

import Prism.framework.*;
import Prism.conn.basic.*;
import Prism.conn.distribution.bottom.*;
import Prism.conn.distribution.top.*;
import Prism.util.*;

import tdspp.Clock.*;
import tdspp.DeploymentAdvisor.*;
import tdspp.Map.*;
import tdspp.RenderingAgent.*;
import tdspp.Repository.*;
import tdspp.ResourceManager.*;
import tdspp.ResourceMonitor.*;
import tdspp.SAKBUI.*;
import tdspp.SimulationAgent.*;
import tdspp.StrategyAnalysisKB.*;
import tdspp.StrategyAnalyzerAgent.*;
import tdspp.util.*;
import tdspp.Weather.*;
import tdspp.WeatherAnalyzer.*;

/**
 *
 * This is the sample main class that executes the application as the General.
 *
 */

public class GeneralServer
{
	/**
	 * The main method
	 */
	public static void main(String argv[]) {
		FIFODispatch disp = new FIFODispatch(100);//size of the message queue
		Scaffold s = new Scaffold(ScaffoldConstants.DISPATCH | ScaffoldConstants.SCHEDULER);
		RoundRobinScheduler sch = new RoundRobinScheduler(disp, 10);//10 threads
		s.setScheduler(sch);
		s.setDispatch(disp);
		ArchitectureAdapter arch=new ArchitectureAdapter("General");
		arch.setScaffold(s);
		System.out.println("Creating the architecture");

		
		Clock clk = new Clock("clock");
		clk.setScaffold(s);

		Connector c1 = new Connector("belowClock");
		c1.setScaffold(s);
		
		Weather wth = new Weather("weatherG");
		wth.setScaffold(s);
		wth.setUpdateParams(0.5, 0.6, 0.3, 0.2, 0.4, 2, 2, 5,3);
		wth.setUIUpdInterval(50);
		wth.setUpdInterval(10);
		
		Repository rep = new Repository("repos");
		if(argv.length > 1) rep.setFilename(argv[1]);
		rep.setScaffold(s);
		
		Connector c2 = new Connector("belowRep");
		c2.setScaffold(s);
		
		Map map = new Map("map");
		map.setScaffold(s);
		
		Connector c3 = new Connector("belowMap");
		c3.setScaffold(s);

		WeatherAnalyzer wthan = new WeatherAnalyzer("weatherAnalysis");
		wthan.setScaffold(s);
		
		ResourceManager resmgr = new ResourceManager("resMgr");
		resmgr.setScaffold(s);

		StrategyAnalysisKB sakb = new StrategyAnalysisKB("SAKB");
		sakb.setScaffold(s);
		

		BottomBorderConnector bbc = new BottomBorderConnector ("BBC", new Integer(argv[0]).intValue(),"Server");
		bbc.setScaffold(s);
		
		
		arch.add(clk);
		arch.add(c1);
		arch.add(wth);		
		arch.add(rep);
		arch.add(c2);
		arch.add(map);
		arch.add(c3);
		arch.add(wthan);
		arch.add(resmgr);
		arch.add(sakb);
		arch.add(bbc);
		
		arch.weld(clk,c1);
		arch.weld(c1,wth);
		arch.weld(c1,rep);
		arch.weld(wth,c2);
		arch.weld(c1,c2);
		arch.weld(rep,c2);
		
		arch.weld(c2,map);
		arch.weld(c2,c3);
		arch.weld(map,c3);
		
		arch.weld(c2,wthan);

		arch.weld(c3,resmgr);
		arch.weld(c3,sakb);
		arch.weld(wthan,bbc);

		arch.weld(c3,bbc);
		arch.weld(resmgr,bbc);
		arch.weld(sakb,bbc);
		
		System.out.println("Architecture looks like " + arch);

		// start it
		sch.start();
		arch.start();
		System.out.println("Architecture started ");
	}

}