package tdspp.apps;

import Prism.core.*;

import tdspp.Clock.*;
import tdspp.DeploymentAdvisor.*;
import tdspp.Map.*;
import tdspp.hq.RenderingAgent.*;
import tdspp.Repository.*;
import tdspp.ResourceManager.*;
import tdspp.ResourceMonitor.*;
import tdspp.SAKBUI.*;
import tdspp.SimulationAgent.*;
import tdspp.StrategyAnalysisKB.*;
import tdspp.StrategyAnalyzerAgent.*;
import tdspp.util.*;
import tdspp.Weather.*;
import tdspp.WeatherAnalyzer.*;

/**
 *
 * This is the sample main class that executes the application as the General.
 *
 */

public class headquartersUI
{
	/**
	 * The main method
	 */
	public static void main(String argv[]) {
		FIFOScheduler sched = new FIFOScheduler(500);//size of the message queue
		Scaffold s = new Scaffold();
		RRobinDispatcher disp = new  RRobinDispatcher(sched, 2);
		s.dispatcher=disp;
		s.scheduler=sched;    
		//DOOVDE
		Architecture arch=new Architecture("General");
		arch.scaffold=s;
		System.out.println("Creating the architecture");

		Clock clk = new Clock("clock");
		Component clkC=new Component("clock");
		clkC.setImplementation( clk);
		clkC.scaffold=s;
		arch.add(clkC);

		

		Weather wth = new Weather("weatherG");
		Component wthC=new Component("weatherG");
		wthC.setImplementation(wth);
		wthC.scaffold=s;
		wth.setUpdateParams(0.5, 0.6, 0.3, 0.2, 0.4, 2, 2, 5,3);
		wth.setUIUpdInterval(50);
		wth.setUpdInterval(10);
		arch.add(wthC);

		Repository rep = new Repository("repos");
		if(argv.length > 2) rep.setFilename(argv[3]);
		Component repC=new Component("repos");
		repC.setImplementation( rep);
		repC.scaffold=s;
		arch.add(repC);


		
		Map map = new Map("map");
		Component mapC=new Component("map");
		mapC.setImplementation( map);
		mapC.scaffold=s;
		arch.add(mapC);

		
		WeatherAnalyzer wthan = new WeatherAnalyzer("weatherAnalysis");
		Component wthanC=new Component("weatherAnalysis");
		wthanC.setImplementation( wthan);
		wthanC.scaffold=s;
		arch.add(wthanC);


		ResourceManager resmgr = new ResourceManager("resMgr");
		Component resmgrC=new Component("resMgr");
		resmgrC.setImplementation( resmgr);
		resmgrC.scaffold=s;
		arch.add(resmgrC);


		StrategyAnalysisKB sakb = new StrategyAnalysisKB("SAKB");
		Component sakbC=new Component("SAKB");
		sakbC.setImplementation( sakb);
		sakbC.scaffold=s;
		arch.add(sakbC);


 

		SimulationAgent simag = new SimulationAgent("SimAgt");
		Component simagC=new Component("SimAgt");
		simagC.setImplementation( simag);
		simagC.scaffold=s;
		arch.add(simagC);

		
		ResourceMonitor resmon = new ResourceMonitor("ResMon");
		Component resmonC=new Component("ResMon");
		resmonC.setImplementation( resmon);
		resmonC.scaffold=s;
		arch.add(resmonC);

		
		StrategyAnalyzerAgent saa = new StrategyAnalyzerAgent("SAAgent");
		Component saaC=new Component("SAAgent");
		saaC.setImplementation( saa);
		saaC.scaffold=s;
		arch.add(saaC);

		
		DeploymentAdvisor da = new DeploymentAdvisor("depAdv");
		Component daC=new Component("depAdv");
		daC.setImplementation( da);
		daC.scaffold=s;
		arch.add(daC);

		
		SAKBUI sakbui = new SAKBUI("sakbUI");
		Component sakbuiC=new Component("sakbUI");
		sakbuiC.setImplementation( sakbui);
		sakbuiC.scaffold=s;
		arch.add(sakbuiC);

		
		
		tdspp.hq.RenderingAgent.RenderingAgent ui = new tdspp.hq.RenderingAgent.RenderingAgent("HQUI");
		Component uiC=new Component("HQUI");
		uiC.setImplementation( ui);
		uiC.scaffold=s;
		arch.add(uiC);

		Port p1;
		Port p2;				
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( clkC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(wthC);
		clkC.addCompPort(p1);
		wthC.addCompPort(p2);
		arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( clkC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(simagC);
		clkC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
	    p1.setParentBrick( clkC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
	    p2.setParentBrick(resmonC);
	    clkC.addCompPort(p1);
	    resmonC.addCompPort(p2);
	    arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
	    p1.setParentBrick( clkC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
	    p2.setParentBrick(sakbC);
	    clkC.addCompPort(p1);
		sakbC.addCompPort(p2);
	    arch.weld(p1,p2);
	    //clock done
		p1=new Port("rep",PrismConstants.REQUEST);
		p1.setParentBrick( repC);
		p2=new Port("req",PrismConstants.REPLY);
		p2.setParentBrick(sakbC);
		repC.addCompPort(p1);
		sakbC.addCompPort(p2);
		arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( repC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(simagC);
		repC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( repC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(uiC);
		repC.addCompPort(p1);
		uiC.addCompPort(p2);
		arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( repC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(resmgrC);
		repC.addCompPort(p1);
		resmgrC.addCompPort(p2);
		arch.weld(p1,p2);

		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( repC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(mapC);
		repC.addCompPort(p1);
		mapC.addCompPort(p2);
		arch.weld(p1,p2);
	    
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( wthC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(wthanC);
		wthC.addCompPort(p1);
		wthanC.addCompPort(p2);
		arch.weld(p1,p2);		
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( wthC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(mapC);
		wthC.addCompPort(p1);
		mapC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( wthC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(uiC);
		wthC.addCompPort(p1);
		uiC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( mapC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(simagC);
		mapC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( mapC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(sakbC);
		mapC.addCompPort(p1);
		sakbC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( mapC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(resmgrC);
		mapC.addCompPort(p1);
		resmgrC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( mapC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(uiC);
		mapC.addCompPort(p1);
		uiC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( mapC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(simagC);
		mapC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( simagC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(mapC);
		mapC.addCompPort(p2);
		simagC.addCompPort(p1);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( sakbC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(simagC);
		sakbC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( sakbC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(uiC);
		sakbC.addCompPort(p1);
		uiC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( sakbC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(sakbuiC);
		sakbC.addCompPort(p1);
		sakbuiC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( sakbC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(saaC);
		sakbC.addCompPort(p1);
		saaC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REPLY);
		p1.setParentBrick( sakbC);
		p2=new Port("reqClk",PrismConstants.REQUEST);
		p2.setParentBrick(daC);
		sakbC.addCompPort(p1);
		daC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(resmgrC);
		uiC.addCompPort(p1);
		resmgrC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(simagC);
		uiC.addCompPort(p1);
		simagC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(resmonC);
		uiC.addCompPort(p1);
		resmonC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(daC);
		uiC.addCompPort(p1);
		daC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(saaC);
		uiC.addCompPort(p1);
		saaC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( uiC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(sakbuiC);
		uiC.addCompPort(p1);
		sakbuiC.addCompPort(p2);
		arch.weld(p1,p2);
		
		p1=new Port("repClk",PrismConstants.REQUEST);
		p1.setParentBrick( resmonC);
		p2=new Port("reqClk",PrismConstants.REPLY);
		p2.setParentBrick(resmgrC);
		resmonC.addCompPort(p1);
		resmgrC.addCompPort(p2);
		arch.weld(p1,p2);
		
		disp.start();
		arch.start();
	}

}