package tdspp.hq.RenderingAgent;

import java.util.*;

import Prism.core.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import tdspp.Clock.*;
import tdspp.DeploymentAdvisor.*;
import tdspp.Map.*;
import tdspp.Repository.*;
import tdspp.ResourceManager.*;
import tdspp.ResourceMonitor.*;
import tdspp.SAKBUI.*;
import tdspp.SimulationAgent.*;
import tdspp.StrategyAnalysisKB.*;
import tdspp.StrategyAnalyzerAgent.*;
import tdspp.util.*;
//import tdspp.hq.util.*;
import tdspp.Weather.*;
import tdspp.WeatherAnalyzer.*;


/**
 * This is the main rendering agent that shows the interface to the user.
 *
 * This being the main GUI, virtually all exposed actions go through this agent. Thus it just
 * acts as a gateway for all that functionality.
 *
 * <table border=0>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Required, Active, User driven </td>
 * </table>
 *
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 *
*/

public class RenderingAgentImp extends GenericImp {
	/**
	 * The main frame
	 */
	MainFrame frm;

	/**
	 * The Regions data
	 */
	Region[][] regions;

	/**
	 * The Image data
	 */
	TDSImage tdsImage[];

	/**
	 * GUI mode
	 */
	public short modeGUI[];

	/**
	 * Flag if previous operation move
	 */
	public boolean moveFlag[];

	/**
	 * Incomplete message (request);
	 */
	Prism.core.Event incompleteReq[];

	public int nextToShow;

	public int nextToAdvice;

	public String messageToSend;

	/**
	 * Constructor
	 *
	*/
	public RenderingAgentImp(Prism.core.AbstractImplementation parent) {
		super(parent);

		frm = null;
		messageToSend = new String("");
		regions = new Region[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS][];
		incompleteReq = new Prism.core.Event[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		moveFlag = new boolean[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		modeGUI = new short[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		tdsImage = new TDSImage[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		nextToShow = 0;
		nextToAdvice = 0;

		for(int i = 0; i < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS; i++) {
			modeGUI[i] = HQConstants.RENDERINGAGENT_MODE_GETTING;
			moveFlag[i] = false;
		}

	}


	/**
	 * set region data for the map
	 */
	public void setRegions(int subMapId, Region[] newRegions) {
		this.regions[subMapId] = newRegions;
	}

	/**
	 * set image data for the map
	 */
	public void setTDSImage(int subMapId, TDSImage tdsImg) {

		this.tdsImage[subMapId] = new TDSImage();
		this.tdsImage[subMapId].inputName = tdsImg.inputName;

		this.tdsImage[subMapId].outputName = tdsImg.outputName;

		this.tdsImage[subMapId].img = tdsImg.img;

	}

	/**
	 * refresh view
	 */
	public void refresh(int subMapId) {
		frm.tp[subMapId].invalidate();
		frm.tp[subMapId].repaint();
		for(int i = 0; i < HQConstants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for(int j = 0; j < HQConstants.REPOSITORY_CELLS_PER_SUBMAP; j++) {
				Region reg = regions[subMapId][(i * HQConstants.REPOSITORY_CELLS_PER_SUBMAP) + j];
				frm.tp[subMapId].drawIcons(reg,null,true);
			}
	}

	/**
	 * Show deployment advice
	 */
	public synchronized void showDeployAdvice(Vector regionVec, Vector levelVec) {
		System.out.println("drew" +  nextToAdvice );
		this.frm.tp[nextToAdvice].showDeployAdvice(regionVec, levelVec);

		this.modeGUI[nextToAdvice] = HQConstants.RENDERINGAGENT_MODE_DEPLOYADV;
		nextToAdvice++;
		if(nextToAdvice < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS) {
			System.out.println("inside" +  nextToAdvice );
			Prism.core.Event req = new Prism.core.Event(messageToSend);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,regions[nextToAdvice]);
			parent.send(req);
		}
		else
			nextToAdvice++;

		this.frm.bp.updateGUIMode();
	}


	/**
	 * Show simulation UI
	 */
	public void showSimulation(int subMapId, boolean startSim) {
		this.frm.bp.selectedSubMapId = subMapId;
		if(startSim) {
			if(this.frm.bp.isModeGUI(HQConstants.RENDERINGAGENT_MODE_STARTSIM)) {
				this.frm.bp.setModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMULATION);
				this.frm.bp.updateGUIMode();
			}
		} else {
			this.frm.bp.setModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMACCEPT);
			this.frm.bp.updateGUIMode();
			//System.out.println("mode" + "  " + this.modeGUI[subMapId] + "  i  " + subMapId );
		}
	}

	/**
	 * Shows the map of a particular area by contacting the Map component. It gets the infomation from
	 * the Map and tries to render it onto the screen.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	MapImp#getMap
	 * @return	void
	 *
	*/
	public void showMap(Region r) {
	}

	/**
	 * Shows the map with weather information of a particular area by contacting the Map
	 * and the Weather components. It gets the infomation from these two components.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	WeatherAnalysisImp#getWeather
	 * @return	void
	 *
	*/
	public void showWeather(Region r) {
	}

	/**
	 * Shows the strategy by contacting the StrategyAnalysisKB component.
	 * Called by: None
	 *
	 * @param r	Region handle
	 * @see 	StrategyAnalysisKBImp#strategyFor
	 * @return	void
	 *
	*/
	public void showStrategy(Region r) {
	}

	/**
	 * Gives the information about the available resources.
	 * Called by: None
	 *
	 * @see 	ResourceManagerImp#availableResources
	 * @return	void
	 *
	*/
	public void availableTroops() {
	}

	/**
	 * Analyzes a particular situation or strategy.
	 * Called by: None
	 *
	 * @return	void
	 *
	*/
	public void analyzeSituation() {
	}

	/**
	 * Create and show the GUI
	 * Called by: None
	 *
	 * @return	void
	 *
	*/
	public void show() {
		if(frm == null)
			frm = new MainFrame("TDS++ Headquarters",this);
		while(nextToShow < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS) {
			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_GETMAP);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID,new Integer(nextToShow));
			parent.send(req);
			nextToShow++;
		}
	}

	/**
	 * Save the request for move resources
	 * Called by: None
	 *
	 * @return	void
	 *
	*/

	public void saveRequest(Prism.core.Event r, int subMapId) {
		this.incompleteReq[subMapId] = r;
	}
}

/**
 * The main frame displayed by the RenderingAgent. The frame consists of two panels.
 * The top panel shows the map and lower panel shows the options available.
 * This is defined as a local class as it is used only by the RenderingAgent.
 */

class MainFrame extends Frame implements WindowListener {
	/**
	 * Parent Rendering Agent.
	 */
	RenderingAgentImp parent;

	/**
	 * Currently selected cell x
	 */
	int cellX;

	/**
	 * Currently selected cell y
	 */
	int cellY;

	/**
	 * The top panel
	 */
	public TopPanel tp[];

	/**
	 * The bottom panel
	 */
	public BottomPanel bp;

	public int subMapId;




	/**
	 * Constructor
	 */
	public MainFrame(String name, RenderingAgentImp par) {
		super(name);
		parent = par;
		cellX = cellY = -1;
		setSize(HQConstants.MAINFRAME_HEIGHT,HQConstants.MAINFRAME_HEIGHT);
		setResizable(false);
		/*
		GridLayout 	grid = new GridLayout(5,4);
		setLayout(grid);*/
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);
		addWindowListener(this);
		tp = new TopPanel[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		for(int i = 0; i < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS; i++) {

			c.gridx = i % 4;
			c.gridy = i / 4;

			tp[i] = new TopPanel(parent,this);
			tp[i].subMapId = i;

			tp[i].setSize(HQConstants.MAINFRAME_SUBMAP,HQConstants.MAINFRAME_SUBMAP);
			gridbag.setConstraints(tp[i],c);
			add(tp[i]);
			//System.out.println("top panel "+i);

		}

		bp = new BottomPanel(parent,this);
		c.gridx = 0;
		c.gridy = 5;
		c.gridheight = GridBagConstraints.REMAINDER;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(bp,c);
		//bp.setSize(HQConstants.MAINFRAME_WIDTH,HQConstants.MAINFRAME_HEIGHT - HQConstants.MAINFRAME_WIDTH - HQConstants.MAINFRAME_CAPTION_HEIGHT);
		add(bp);
		System.out.println("bottom panel completed");
		System.out.println("HQConstants.MAINFRAME_SUBMAP >>>>>>>>>>>>>>" + HQConstants.MAINFRAME_SUBMAP);
		doLayout();
		show();
	}



	/**
	 * Window closing function
	 */
	public void windowClosing(WindowEvent e) {
		dispose();
	}

	/**
	 * Window activated
	 */
	public void windowActivated(WindowEvent e) {
	}

	/**
	 * Window closed
	 */
	public void windowClosed(WindowEvent e) {
		System.exit(0);
	}

	/**
	 * Window deactivated
	 */
	public void windowDeactivated(WindowEvent e) {
	}

	/**
	 * Window iconified
	 */
	public void windowIconified(WindowEvent e) {
	}

	/**
	 * Window opened
	 */
	public void windowOpened(WindowEvent e) {
	}

	/**
	 * Window deiconified
	 */
	public void windowDeiconified(WindowEvent e) {
	}

}

/**
 * The top panel
 * This is defined as a local class as it is used only by the RenderingAgent.
 */

class TopPanel extends Panel implements ImageObserver {
	/**
	 * Text shown on the screen.
	 */
	public String text;

	/**
	 * Parent Rendering Agent.
	 */
	RenderingAgentImp parent;

	/**
	 * The parent frame
	 */
	MainFrame frm;

	/**
	 * Image error occurred
	 */
	boolean imageError[];

	int subMapId;

	public int cellSide;
	/**
	 * Constructor
	 */
	public TopPanel(RenderingAgentImp par, MainFrame pfrm) {
		super();
		parent = par;
		frm = pfrm;
		text = "Fetching data";
		cellSide = 0;
		imageError = new boolean[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS];
		for(int i = 0; i < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS; i++) {
			imageError[i] = false;
		}
		setLayout(null);

		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				if(parent.regions[subMapId] == null) return;

				int x = me.getX();
				int y = me.getY();
				int oldX = TopPanel.this.frm.cellX;
				int oldY = TopPanel.this.frm.cellY;

				x /= TopPanel.this.cellSide;
				y /= TopPanel.this.cellSide;



				Graphics g = TopPanel.this.getGraphics();
				if(oldX != -1) TopPanel.this.paintCell(oldX,oldY,g);
				TopPanel.this.frm.bp.selectedSubMapId = subMapId;
				TopPanel.this.frm.cellX = x;
				TopPanel.this.frm.cellY = y;
				TopPanel.this.paintCell(x,y,g);

				int fromSubMap = getFromSubMap();

				if(fromSubMap != -1) {

					TopPanel.this.parent.moveFlag[fromSubMap] = false;
					Region regTo = TopPanel.this.parent.regions[subMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
					//System.out.println("adding parameter...." + regTo);
					TopPanel.this.parent.incompleteReq[fromSubMap].addParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_TOREGION,regTo);
					parent.parent.send(TopPanel.this.parent.incompleteReq[fromSubMap]);
					//System.out.println("populating");
					Region regFrom = (Region)TopPanel.this.parent.incompleteReq[fromSubMap].getParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_FROMREGION);
					doMove(regFrom,regTo,(String [])TopPanel.this.parent.incompleteReq[fromSubMap].getParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_RESOURCENAMES));

				}
			}

			private int getFromSubMap() {
				for(int i = 0; i < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS ; i++) {
					if(TopPanel.this.parent.moveFlag[i])
						return i;
				}
				return -1;

			}
			});
	}



	private void doMove(Region regFrom, Region regTo, String[] resourceNames) {
		//System.out.println("in do Move ......." + resourceNames.length);
		for(int i = 0; i < resourceNames.length; i++) {
			if(resourceNames[i] == null)
				break;
			if(resourceNames[i].equalsIgnoreCase(HQConstants.ARMYRESOURCESNAMES[0])){
					regFrom.armyRes.removeInfantry();
					regTo.armyRes.setInfantry();
			}
			if(resourceNames[i].equalsIgnoreCase(HQConstants.ARMYRESOURCESNAMES[1])){
					regFrom.armyRes.removeTank();
					regTo.armyRes.setTank();
			}
			if(resourceNames[i].equalsIgnoreCase(HQConstants.ARMYRESOURCESNAMES[2])){
					regFrom.armyRes.removeMineField();
					regTo.armyRes.setMineField();
			}
			if(resourceNames[i].equalsIgnoreCase(HQConstants.ARMYRESOURCESNAMES[3])){
					regFrom.armyRes.removeEnemyInfantry();
					regTo.armyRes.setEnemyInfantry();
			}
			if(resourceNames[i].equalsIgnoreCase(HQConstants.ARMYRESOURCESNAMES[4])){
					regFrom.armyRes.removeEnemyTank();
					regTo.armyRes.setEnemyTank();
			}
			updateCell(regFrom);
			updateCell(regTo);
		}
	}

	/**
	 * paint cell
	 */
	void paintCell(int i, int j, Graphics g) {


		int x = 0, y = 0;
		// g.setColor(Color.white);
		// g.fillRect(x + (i * HQConstants.MAINFRAME_CELL_SIDE),y + (j * HQConstants.MAINFRAME_CELL_SIDE),HQConstants.MAINFRAME_CELL_SIDE,HQConstants.MAINFRAME_CELL_SIDE);
		g.setColor(Color.black);
		//g.clearRect(x + (i * HQConstants.MAINFRAME_CELL_SIDE),y + (j * HQConstants.MAINFRAME_CELL_SIDE),HQConstants.MAINFRAME_CELL_SIDE,HQConstants.MAINFRAME_CELL_SIDE);

		cellSide = this.getSize().height / HQConstants.REPOSITORY_CELLS_PER_SUBMAP;

		g.drawRect(x + (i * cellSide), y + (j * cellSide), cellSide, cellSide);

		Region reg = parent.regions[subMapId][(i * HQConstants.REPOSITORY_CELLS_PER_SUBMAP) + j];
		if(frm.cellX == i && frm.cellY == j) {
			g.setColor(Color.white);
			g.setXORMode(Color.black);
			//to be done !!!
			//g.drawRect(x + (i * HQConstants.MAINFRAME_CELL_SIDE)+1,y + (j * HQConstants.MAINFRAME_CELL_SIDE) + 1,HQConstants.MAINFRAME_CELL_SIDE-2,HQConstants.MAINFRAME_CELL_SIDE-2);
			g.setPaintMode();
		}
		drawIcons(reg,g,false);

	}

	/**
	 * The paint method
	 */
	public void paint(Graphics g) {
		super.paint(g);

		if(parent.regions[subMapId] == null) {
			g.drawString(text,(this.getSize().width-50)/2,this.getSize().height/2);
		} else {
			// draw image
			//if(subMapId == 0)
			//System.out.println("Map id : " + subMapId + "imageError" + prepareImage(parent.tdsImage[subMapId].img , -1, -1, this));
			if(!imageError[subMapId] && parent.tdsImage[subMapId].img != null && prepareImage(parent.tdsImage[subMapId].img , -1, -1, this)) {

			//	System.out.println("panel number" + this.subMapId + "submappId" + subMapId + "image  " + parent.tdsImage[subMapId].outputName);
				g.drawImage(parent.tdsImage[subMapId].img ,0,0,this);
			}
			for(int i = 0; i < HQConstants.REPOSITORY_CELLS_PER_SUBMAP; i++)
				for(int j = 0; j < HQConstants.REPOSITORY_CELLS_PER_SUBMAP; j++)
					paintCell(i,j,g);
		}
		//drawMovers(HQConstants.TOPPANEL_ARROW_ALL,g);
	}

	/**
	 * update a particular cell
	 */
	public void updateCell(Region reg) {
		Graphics g = this.getGraphics();
		paintCell(reg.x,reg.y,g);
		Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTR);
		req.eventType=PrismConstants.REQUEST;
		req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_SETATTR_REGION,reg);
		parent.parent.send(req);
	}

	/**
	 * update screen
	 */
	public void update(Graphics g) {
		paint(g);
	}

	/**
	 * Image update routine
	 */
	public boolean imageUpdate(Image img, int flags, int x, int y, int width, int height) {

		if(img != parent.tdsImage[subMapId].img) return true;
		if((flags & SOMEBITS) != 0) {
			repaint(x, y, width, height);
		}
		else if((flags & ABORT) != 0) {
			imageError[subMapId] = true;
			repaint();
		}
		return (flags & (ALLBITS|ABORT)) == 0;
	}

	/**
	 * Draw Icons for the resources
	 */
	public boolean drawIcons(Region reg, Graphics g, boolean erase) {
		int x = 0, y = 0;
		if(reg.armyRes.isInfantry()) drawFriendlyTroop(x + (reg.x * cellSide)+ HQConstants.TOPPANEL_TROOPOFFSET,y + (reg.y * cellSide) + HQConstants.TOPPANEL_TROOPOFFSET, reg.armyRes.getTotalState(), erase);
		if(reg.armyRes.isEnemyInfantry()) drawEnemyTroop(x + (reg.x * cellSide) + HQConstants.TOPPANEL_TROOPOFFSET,y + (reg.y * cellSide) + HQConstants.TOPPANEL_TROOPOFFSET, reg.armyRes.getTotalState(), erase);
		if(reg.armyRes.isTank()) drawFriendlyTank (x + (reg.x * cellSide) + HQConstants.TOPPANEL_TANKOFFSET,y + (reg.y * cellSide) + HQConstants.TOPPANEL_TANKOFFSET, reg.armyRes.getTotalState(), erase);
		if(reg.armyRes.isEnemyTank()) drawEnemyTank (x + (reg.x * cellSide) + HQConstants.TOPPANEL_TANKOFFSET,y + (reg.y * cellSide) + HQConstants.TOPPANEL_TANKOFFSET, reg.armyRes.getTotalState(), erase);
		if(reg.armyRes.isMineField()) drawMineField(x + (reg.x * cellSide) + HQConstants.TOPPANEL_MINEOFFSETX,y + (reg.y * cellSide) + HQConstants.TOPPANEL_MINEOFFSETY, reg.armyRes.getTotalState(), erase);
		return true;
	}


	/**
	 * Draw Friendly Troop Icon
	 */
	private void drawFriendlyTroop(int pivotX, int pivotY, float color, boolean erase) {
		Graphics g = this.getGraphics();
		//color = 1;
		g.setColor(new Color(0,HQConstants.COLOR_RED, color));
		if(erase) g.setXORMode(new Color(255,255,128));
		g.drawOval(pivotX,pivotY, HQConstants.TOPPANEL_TROOPWIDTH-1, HQConstants.TOPPANEL_TROOPHEIGHT-1);
		g.fillOval(pivotX,pivotY, HQConstants.TOPPANEL_TROOPWIDTH-1, HQConstants.TOPPANEL_TROOPHEIGHT-1);

		int verticesX[] = new int [HQConstants.TOPPANEL_TRIANGLE];
		int verticesY[] = new int [HQConstants.TOPPANEL_TRIANGLE];

		verticesX[0] = pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2;
		verticesY[0] = pivotY + HQConstants.TOPPANEL_TROOPHEIGHT;

		verticesX[1] = pivotX;
		verticesY[1] = pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT;

		verticesX[2] = pivotX + HQConstants.TOPPANEL_TROOPWIDTH;
		verticesY[2] = pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT;

		g.drawLine(pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2,pivotY + HQConstants.TOPPANEL_TROOPHEIGHT, pivotX, pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT);
		g.drawLine(pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2,pivotY + HQConstants.TOPPANEL_TROOPHEIGHT, pivotX + HQConstants.TOPPANEL_TROOPWIDTH, pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT);
		g.drawLine(pivotX,pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT, pivotX + HQConstants.TOPPANEL_TROOPWIDTH, pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT);

		g.fillPolygon(verticesX,verticesY,HQConstants.TOPPANEL_TRIANGLE);

	}

	/**
	 * Draw Friendly Tank Icon
	 */
	private void drawFriendlyTank(int pivotX, int pivotY, float color, boolean erase) {
		//tank
		//color = 1;
		Graphics g = this.getGraphics();
		g.setColor(new Color(0,HQConstants.COLOR_RED, color));
		if(erase) g.setXORMode(new Color(255,255,128));
		g.drawRect(pivotX,pivotY, HQConstants.TOPPANEL_TANKWIDTH, HQConstants.TOPPANEL_TANKHEIGHT);
		g.fillRect(pivotX,pivotY, HQConstants.TOPPANEL_TANKWIDTH, HQConstants.TOPPANEL_TANKHEIGHT);
	}


	/**
	 * Draw Enemy Troop Icon
	 */
	private void drawEnemyTroop(int pivotX, int pivotY, float color, boolean erase) {
		//face
		//color = 1;
		Graphics g = this.getGraphics();
		g.setColor(Color.red);
		if(erase) g.setXORMode(new Color(255,255,128));
		//System.out.println((pivotX-HQConstants.TOPPANEL_TROOPWIDTH/2) + " " + (pivotY-HQConstants.TOPPANEL_TROOPHEIGHT/2) + " " + HQConstants.TOPPANEL_TROOPWIDTH + " " + HQConstants.TOPPANEL_TROOPHEIGHT);


		g.drawOval(pivotX,pivotY, HQConstants.TOPPANEL_TROOPWIDTH-1, HQConstants.TOPPANEL_TROOPHEIGHT-1);
		g.fillOval(pivotX,pivotY, HQConstants.TOPPANEL_TROOPWIDTH-1, HQConstants.TOPPANEL_TROOPHEIGHT-1);

		g.drawLine(pivotX,pivotY + HQConstants.TOPPANEL_TROOPHEIGHT, pivotX + HQConstants.TOPPANEL_TROOPWIDTH, pivotY + HQConstants.TOPPANEL_TROOPHEIGHT);

		g.drawLine(pivotX, pivotY + HQConstants.TOPPANEL_TROOPHEIGHT, pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2, pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT);
		g.drawLine(pivotX + HQConstants.TOPPANEL_TROOPWIDTH,pivotY + HQConstants.TOPPANEL_TROOPHEIGHT, pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2, pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT);

		int verticesX[] = new int [HQConstants.TOPPANEL_TRIANGLE];
		int verticesY[] = new int [HQConstants.TOPPANEL_TRIANGLE];

		verticesX[0] = pivotX;
		verticesY[0] = pivotY + HQConstants.TOPPANEL_TROOPHEIGHT;

		verticesX[1] = pivotX + HQConstants.TOPPANEL_TROOPWIDTH;
		verticesY[1] = pivotY + HQConstants.TOPPANEL_TROOPHEIGHT;

		verticesX[2] = pivotX + HQConstants.TOPPANEL_TROOPWIDTH/2;
		verticesY[2] = pivotY + 2*HQConstants.TOPPANEL_TROOPHEIGHT;



		g.fillPolygon(verticesX,verticesY,HQConstants.TOPPANEL_TRIANGLE);
	}

	/**
	 * Draw Enemy Tank Icon
	 */
	private void drawEnemyTank(int pivotX, int pivotY, float color, boolean erase) {
		//tank
		//color = 1;
		Graphics g = this.getGraphics();
		g.setColor(Color.red);
		if(erase) g.setXORMode(new Color(255,255,128));
		g.drawRect(pivotX,pivotY, HQConstants.TOPPANEL_TANKWIDTH, HQConstants.TOPPANEL_TANKHEIGHT);
		g.fillRect(pivotX,pivotY, HQConstants.TOPPANEL_TANKWIDTH, HQConstants.TOPPANEL_TANKHEIGHT);
	}


	/**
	 * Draw Mine Field Icon
	 */
	private void drawMineField(int pivotX, int pivotY, float color, boolean erase) {
		//face
		//color = 1;
		Graphics g = this.getGraphics();
		g.setColor(new Color(0,HQConstants.COLOR_RED,color));
		if(erase) g.setXORMode(new Color(255,255,128));
		g.drawOval(pivotX,pivotY, HQConstants.TOPPANEL_MINERADIUS, HQConstants.TOPPANEL_MINERADIUS);
		g.fillOval(pivotX,pivotY, HQConstants.TOPPANEL_MINERADIUS, HQConstants.TOPPANEL_MINERADIUS);

	}

	/**
	 * Show deployment advice
	 */
	public void showDeployAdvice(Vector regionVec, Vector levelVec) {
		int x = 0, y = 0;
		Graphics g = this.getGraphics();
		for(Enumeration e = regionVec.elements(), le = levelVec.elements(); e.hasMoreElements() && le.hasMoreElements(); ) {
			int val = ((Integer)e.nextElement()).intValue();
			int i = val / HQConstants.REPOSITORY_CELLS_PER_SUBMAP;
			int j = val % HQConstants.REPOSITORY_CELLS_PER_SUBMAP;
			int pri = ((Integer)le.nextElement()).intValue();
			if(pri >= 60) g.setColor(Color.magenta);
			if(pri >= 80) g.setColor(Color.green);
			if(pri <= 40) g.setColor(Color.pink);
			if(pri <= 20) g.setColor(Color.red);
			System.out.println("Dreq Rect" + subMapId);
			//g.drawRect(x + (i * HQConstants.MAINFRAME_CELL_SIDE)+2,y + (j * HQConstants.MAINFRAME_CELL_SIDE) + 2,HQConstants.MAINFRAME_CELL_SIDE-4,HQConstants.MAINFRAME_CELL_SIDE-4);
			//g.drawRect(x + (i * HQConstants.MAINFRAME_CELL_SIDE)+3,y + (j * HQConstants.MAINFRAME_CELL_SIDE) + 3,HQConstants.MAINFRAME_CELL_SIDE-6,HQConstants.MAINFRAME_CELL_SIDE-6);
			g.drawRect(x + (i * cellSide)+2,y + (j * cellSide) + 2,cellSide-4,cellSide-4);
			g.drawRect(x + (i * cellSide)+3,y + (j * cellSide) + 3,cellSide-6,cellSide-6);
		}
	}

}

/**
 * The bottom panel
 * This is defined as a local class as it is used only by the RenderingAgent.
 */

class BottomPanel extends Panel {
	/**
	 * Parent Rendering Agent.
	 */
	RenderingAgentImp parent;

	/**
	 * The parent frame
	 */
	MainFrame frm;

	/**
	 * Buttons
	 */
	Button buttons[];

	/**
	 * Menus
	 */
	PopupMenu popupMenus[];

	int selectedSubMapId;
	/**
	 * Constructor
	 */
	public BottomPanel(RenderingAgentImp par, MainFrame pfrm) {
		super();
		parent = par;
		frm = pfrm;
		selectedSubMapId = 0;
		// count buttons
		int noOfButtons = HQConstants.BOTTOMPANEL_BUTTON_LABELS.length;
		buttons = new Button[noOfButtons];
		// create popup menus
		popupMenus = new PopupMenu[noOfButtons];
		try {
			Class paramTypes[] = { this.getClass() };
			Object params[] = { this };
			java.lang.reflect.Constructor c;
			for(int i = 0; i < noOfButtons; i++) {
				popupMenus[i] = new PopupMenu();
				for(int j = 0; j < HQConstants.BOTTOMPANEL_MENUS[i].length; j++) {
					MenuItem mi = new MenuItem(HQConstants.BOTTOMPANEL_MENUS[i][j]);
					c = ((Class) Class.forName(HQConstants.BOTTOMPANEL_ACTIONS[i][j])).getConstructor(paramTypes);
					mi.addActionListener((ActionListener)c.newInstance(params));
					popupMenus[i].add(mi);
				}
				add(popupMenus[i]);
			}
		}catch(Exception e) {
			e.printStackTrace();
			System.out.println(e);
		}
		// create buttons
		for(int i = 0; i < noOfButtons; i++) {
			buttons[i] = new Button(HQConstants.BOTTOMPANEL_BUTTON_LABELS[i]);
			buttons[i].setSize(HQConstants.MAINFRAME_SUBMAP/noOfButtons,HQConstants.MAINFRAME_HEIGHT - HQConstants.MAINFRAME_WIDTH - HQConstants.MAINFRAME_CAPTION_HEIGHT);
			buttons[i].addActionListener(new ActionListener () {
				public void actionPerformed(ActionEvent ev) {
					if(ev.getSource() instanceof Button) {
						Button src = (Button) ev.getSource();
						if((isModeGUI(HQConstants.RENDERINGAGENT_MODE_GETTING) || isModeGUI(HQConstants.RENDERINGAGENT_MODE_DEPLOYADV)|| isModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMULATION) )&& src.getLabel().equals(HQConstants.BOTTOMPANEL_BUTTON_CANCEL)) {
							for(int tpIndex=0;tpIndex<HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS;tpIndex++)
								BottomPanel.this.frm.tp[tpIndex].repaint();

							setModeGUI(HQConstants.RENDERINGAGENT_MODE_NORMAL);
								BottomPanel.this.updateGUIMode();
						} else if(isModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMACCEPT)) {
							if(src.getLabel().equals(HQConstants.BOTTOMPANEL_BUTTON_ACCEPT)) {
							} else if(src.getLabel().equals(HQConstants.BOTTOMPANEL_BUTTON_REJECT)) {
								RenderingAgentImp ra = BottomPanel.this.frm.parent;
								Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_DONTSAVEFIGHT);
								req.eventType=PrismConstants.REQUEST;
								ra.parent.send(req);
							}
							for(int tpIndex=0;tpIndex<16;tpIndex++) {
								BottomPanel.this.frm.tp[tpIndex].repaint();
							}
							for(int j=0;j<buttons.length;j++) {
								buttons[j].setVisible(true);
								buttons[j].setLabel(HQConstants.BOTTOMPANEL_BUTTON_LABELS[j]);
							}
							setModeGUI(HQConstants.RENDERINGAGENT_MODE_NORMAL);
						} else {

							for(int index = 0; index < HQConstants.BOTTOMPANEL_BUTTON_LABELS.length; index++) {
								if(src.getLabel().equals(HQConstants.BOTTOMPANEL_BUTTON_LABELS[index])) {
									BottomPanel.this.popupMenus[index].show(src,10,10);
									break;
								}
							}
						}
					}
				}
				});
			add(buttons[i]);
		}
	}

	/**
	 * The paint method
	 */
	public void paint(Graphics g) {
		super.paint(g);
		g.setColor(Color.blue);
		g.drawLine(0,1,HQConstants.MAINFRAME_WIDTH-1,1);
	}

	/**
	 * Show deployment advice
	 */
	public void updateGUIMode() {
		//System.out.println("update GUI mode = " + this.parent.modeGUI);
		if(isModeGUI(HQConstants.RENDERINGAGENT_MODE_DEPLOYADV) ||
		   isModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMULATION) ||
		   isModeGUI(HQConstants.RENDERINGAGENT_MODE_GETTING)) {
			for(int i=0;i<buttons.length;i++)
				buttons[i].setVisible(false);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_CANCELIDX].setLabel(HQConstants.BOTTOMPANEL_BUTTON_CANCEL);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_CANCELIDX].setVisible(true);
		} else if(isModeGUI(HQConstants.RENDERINGAGENT_MODE_SIMACCEPT)) {
			for(int i=0;i<buttons.length;i++)
				buttons[i].setVisible(false);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_ACCEPTIDX].setLabel(HQConstants.BOTTOMPANEL_BUTTON_ACCEPT);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_ACCEPTIDX].setVisible(true);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_REJECTIDX].setLabel(HQConstants.BOTTOMPANEL_BUTTON_REJECT);
			buttons[HQConstants.BOTTOMPANEL_BUTTON_REJECTIDX].setVisible(true);
		} else if(isModeGUI(HQConstants.RENDERINGAGENT_MODE_NORMAL)) {
			buttons[HQConstants.BOTTOMPANEL_BUTTON_CANCELIDX].setLabel(HQConstants.BOTTOMPANEL_BUTTON_LABELS[HQConstants.BOTTOMPANEL_BUTTON_CANCELIDX]);
			for(int j=0;j<buttons.length;j++)
				buttons[j].setVisible(true);
		}
	}

	public boolean isModeGUI(short mode) {

		for(int i = 0; i < BottomPanel.this.frm.parent.modeGUI.length; i++) {
			if(BottomPanel.this.frm.parent.modeGUI[i] != mode) {
				return false;
			}
		}
		return true;

	}


	public void setModeGUI(short mode) {

		for(int i = 0; i < BottomPanel.this.frm.parent.modeGUI.length; i++) {
			BottomPanel.this.frm.parent.modeGUI[i] = mode;
		}
	}


	/**
	 * Action listener classes
	 */
	public class SimulateAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			MainFrame frm = BottomPanel.this.frm;
			if(frm.cellX == -1) return;
			Region reg[] = null;
			System.out.println("selectedSubMapId" + selectedSubMapId);
			reg = BottomPanel.this.parent.regions[selectedSubMapId];


			RenderingAgentImp ra = BottomPanel.this.frm.parent;
/*			Region newRegions[];
			int noOfCells = HQConstants.REPOSITORY_CELLS_PER_SUBMAP * HQConstants.REPOSITORY_CELLS_PER_SUBMAP;
			newRegions = new Region[HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS * noOfCells];
			for(int i = 0; i < HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS; i++) {
				for(int j = 0; j < noOfCells; j++) {
					newRegions[i * noOfCells + j] = ra.regions[i][j];
				}
			}

*/
			setModeGUI(HQConstants.RENDERINGAGENT_MODE_STARTSIM);

			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_SIMULATEFIGHT);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,reg);
			ra.parent.send(req);

		}
	}

	public class DeployAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			System.out.println("DeployAction called");
			RenderingAgentImp ra = BottomPanel.this.frm.parent;
			ra.nextToAdvice = 0;
			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_ADVISEDEPLOYMENT);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,ra.regions[ra.nextToAdvice]);
			ra.parent.send(req);
			ra.messageToSend = tdspp.util.MessageConstants.MESSAGE_NAME_ADVISEDEPLOYMENT;

		}
	}
	public class StrategyAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			System.out.println("StrategyAction called");
			RenderingAgentImp ra = BottomPanel.this.frm.parent;
			ra.nextToAdvice = 0;
			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_ANALYZESTRATEGY);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS,ra.regions[ra.nextToAdvice]);
			ra.parent.send(req);
			ra.messageToSend = tdspp.util.MessageConstants.MESSAGE_NAME_ANALYZESTRATEGY ;
		}
	}

	public class KBAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			RenderingAgentImp ra = BottomPanel.this.frm.parent;
			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_SHOWSAKBUI);
			req.eventType=PrismConstants.REQUEST;
			req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_SHOWSAKBUI_PARENT,BottomPanel.this.frm);
			ra.parent.send(req);
		}
	}

	public class WeatherAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			MainFrame frm = BottomPanel.this.frm;
			if(frm.cellX == -1) return;
			Region reg = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
			String str[] = { "Weather Info", "Temperature : " + reg.weatherInfo.temperature,
							"Humidity    : " + reg.weatherInfo.humidity,
							"WindSpeed   : " + reg.weatherInfo.windSpeed,
							 "Sunlight    : " + reg.weatherInfo.sunlight };
			BottomPanel.DialogBox dlg = BottomPanel.this.new DialogBox("Cell Info",str,null, HQConstants.DIALOG_BOXTYPE_INFO);

			dlg.show();
		}
	}


	public class TempUpdateAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			MainFrame frm = BottomPanel.this.frm;
			if(frm.cellX == -1) return;
			Region reg = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
			BottomPanel.DialogBox dlg = BottomPanel.this.new DialogBox("Cell Info",null,reg,HQConstants.DIALOG_BOXTYPE_UPDATEDATA);
			dlg.show();
		}
	}

	public class MoveAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			MainFrame frm = BottomPanel.this.frm;
			if(frm.cellX == -1) return;

			Region reg = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
			if(!hasResources(reg)) return;
			BottomPanel.DialogBox dlg = BottomPanel.this.new DialogBox("Cell Info",null,reg, HQConstants.DIALOG_BOXTYPE_MOVE);
			dlg.show();
		}
	}
	public class UpdateAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			MainFrame frm = BottomPanel.this.frm;
			if(frm.cellX == -1) return;
			Region reg = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
			BottomPanel.DialogBox dlg = BottomPanel.this.new DialogBox("Cell Info",null,reg,HQConstants.DIALOG_BOXTYPE_UPDATE);
			dlg.show();
		}
	}
	public class LogAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
		}
	}


	public class SaveAction implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			RenderingAgentImp ra = BottomPanel.this.frm.parent;
			Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_SAVESTATE);
			req.eventType=PrismConstants.REQUEST;
			ra.parent.send(req);
		}
	}

	public class DialogBox extends Dialog {
		/**
		 * Checkboxes for army resources
		 */
		Checkbox arr[];
		int arrNum;
		java.awt.Component carr[];

		public DialogBox(String title, String text[], Region reg, int type) {
			super(BottomPanel.this.frm,title,true);
			int items = 4;
			if(text != null) {
				items = text.length;
				setLayout(new GridLayout(items,1));
				Label lbl[] = new Label[items];
				for(int i = 0; i < items; i++) {
					lbl[i] = new Label(text[i]);
					add(lbl[i]);
				}
				if(type == HQConstants.DIALOG_BOXTYPE_INFO) {
					setSize(tdspp.util.Constants.MAINFRAME_WIDTH*6/10,tdspp.util.Constants.MAINFRAME_HEIGHT*9/10);
				}
			}
			if(reg != null) {
				//System.out.println("Type >> " + type);
				if(type == HQConstants.DIALOG_BOXTYPE_UPDATE) {

					int noOfChks = HQConstants.DIALOG_CHECKBOXES_updChk.length + HQConstants.DIALOG_CHECKBOXES_unUpdChk.length;
					items = noOfChks + 1;
					setLayout(new GridLayout(items,1));
					arr = new Checkbox[noOfChks];
					arrNum = noOfChks;
					for(int i=0;i<HQConstants.DIALOG_CHECKBOXES_updChk.length;i++) {
						arr[i] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_updChk[i]);
						if(i == 0 && reg.armyRes.isInfantry())
							arr[i].setState(true);
						if(i == 1 && reg.armyRes.isTank())
							arr[i].setState(true);
						add(arr[i]);
					}
					for(int i=0;i<HQConstants.DIALOG_CHECKBOXES_unUpdChk.length;i++) {
						arr[i+HQConstants.DIALOG_CHECKBOXES_updChk.length] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_unUpdChk[i]);
						if(i == 0 && reg.armyRes.isMineField())
							arr[i+HQConstants.DIALOG_CHECKBOXES_updChk.length].setState(true);
						if(i == 1 && reg.armyRes.isEnemyInfantry())
							arr[i+HQConstants.DIALOG_CHECKBOXES_updChk.length].setState(true);
						if(i == 2 && reg.armyRes.isEnemyTank())
							arr[i+HQConstants.DIALOG_CHECKBOXES_updChk.length].setState(true);
						add(arr[i+HQConstants.DIALOG_CHECKBOXES_updChk.length]);
					}
					Button b = new Button("Update");
					b.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent ev) {
							MainFrame frm = BottomPanel.this.frm;
							if(frm.cellX == -1) return;

							Region regAL = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
							System.out.println("================================");
							System.out.println("regA" + regAL);
							System.out.println("================================");
							int addCounter = 0, removeCounter = 0;
							String addResourceNames[], removeResourceNames[];
							addResourceNames = new String[5];
							removeResourceNames = new String[5];
							int counters[];
							setVisible(false);
							for(int tpIndex=0;tpIndex<HQConstants.REPOSITORY_NUMBER_OF_SUBMAPS;tpIndex++)
								frm.tp[tpIndex].drawIcons(regAL,null,true);
							//System.out.println("bedore readdialogbox");
							counters = readDialogBox(regAL,addResourceNames, addCounter,removeResourceNames, removeCounter);
							//System.out.println("after readdialogbox");
							addCounter = counters[0];
							removeCounter = counters[1];

							if(addCounter > 0) {
								RenderingAgentImp ra = BottomPanel.this.frm.parent;
								Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_UPDATERESOURCES);
								req.eventType=PrismConstants.REQUEST;
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_OPERATIONFLAG, new Integer(1));
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_REGION,regAL);
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_RESOURCENAMES,addResourceNames);
								ra.parent.send(req);
							}

							if(removeCounter > 0) {
								RenderingAgentImp ra = BottomPanel.this.frm.parent;
								Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_UPDATERESOURCES);
								req.eventType=PrismConstants.REQUEST;
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_OPERATIONFLAG, new Integer(-1));
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_REGION,regAL);
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_UPDATERESOURCES_RESOURCENAMES,removeResourceNames);
								ra.parent.send(req);
							}
							dispose();
							//frm.tp[subMapId].updateCell(regAL);
						}
						});
					add(b);
					setSize(tdspp.util.Constants.MAINFRAME_WIDTH*7/8,tdspp.util.Constants.MAINFRAME_HEIGHT*7/8);
				} else if (type == tdspp.util.Constants.DIALOG_BOXTYPE_UPDATEDATA) {

					int noOfChks = tdspp.util.Constants.DIALOG_CHECKBOXES_updData.length;
					int noOfTexts = tdspp.util.Constants.DIALOG_TEXTBOXES_updData.length;
					int noOfLists = tdspp.util.Constants.DIALOG_LISTBOXES_updData.length;
					items = noOfChks + noOfTexts + noOfLists + 1;
					setLayout(new GridLayout(items,2));
					arr = new Checkbox[noOfChks];
					carr = new java.awt.Component[noOfTexts + noOfLists];
					arrNum = noOfChks;
					int done = 0;
					for(int i=0;i<tdspp.util.Constants.DIALOG_CHECKBOXES_updData.length;i++) {
						arr[i] = new Checkbox(tdspp.util.Constants.DIALOG_CHECKBOXES_updData[i]);
						if(i == 0 && reg.armyRes.isEnemyInfantry())
							arr[i].setState(true);
						if(i == 1 && reg.armyRes.isTank())
							arr[i].setState(true);
						add(arr[i]);
						add(new Label(""));
					}
					for(int i=0;i<tdspp.util.Constants.DIALOG_TEXTBOXES_updData.length;i++) {
						carr[i+done] = new TextField(5);
						if(i == 0)
							((TextField)carr[i+done]).setText("" + reg.armyRes.stateAmmunition);
						if(i == 1)
							((TextField)carr[i+done]).setText("" + reg.armyRes.stateFood);
						if(i == 2)
							((TextField)carr[i+done]).setText("" + reg.armyRes.statePopulation);
						if(i == 3)
							((TextField)carr[i+done]).setText("" + reg.weatherInfo.temperature);
						if(i == 4)
							((TextField)carr[i+done]).setText("" + reg.weatherInfo.humidity);
						if(i == 5)
							((TextField)carr[i+done]).setText("" + reg.weatherInfo.windSpeed);
						if(i == 6)
							((TextField)carr[i+done]).setText("" + reg.weatherInfo.sunlight);
						add(new Label(tdspp.util.Constants.DIALOG_TEXTBOXES_updData[i]));
						add(carr[i+done]);
					}
					done += noOfTexts;
					for(int i=0;i<tdspp.util.Constants.DIALOG_LISTBOXES_updData.length;i++) {
						carr[i+done] = new java.awt.List(1);
						if(i == 0) {
							for(int j=0;j<tdspp.util.Constants.DIALOG_TERRAINTYPE_DESC.length;j++) {
								((java.awt.List)carr[i+done]).add(tdspp.util.Constants.DIALOG_TERRAINTYPE_DESC[j]);
								if(reg.terrainDesc.type == tdspp.util.Constants.DIALOG_TERRAINTYPE_VALS[j])
									((java.awt.List)carr[i+done]).select(j);
							}
						 }
						add(new Label(tdspp.util.Constants.DIALOG_LISTBOXES_updData[i]));
						add(carr[i+done]);
					}
					Button b = new Button("Update");
					b.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent ev) {
							MainFrame frm = BottomPanel.this.frm;
							if(frm.cellX == -1) return;
							Region regAL = new Region();
							regAL = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
							setVisible(false);
							frm.tp[selectedSubMapId].drawIcons(regAL,null,true);
							String name;
							if(DialogBox.this.arr != null) {
								for(int i=0; i<arr.length; i++) {
									if(arr[i].getState()) {
										name = new String(arr[i].getLabel());
										//System.out.println("Selected dude" + name + "   " + tdspp.util.Constants.DIALOG_CHECKBOXES_updChk[0]);
										if(name.equalsIgnoreCase(tdspp.util.Constants.DIALOG_CHECKBOXES_updData[0]))
											regAL.armyRes.setEnemyInfantry();
										if(name.equalsIgnoreCase(tdspp.util.Constants.DIALOG_CHECKBOXES_updData[1]))
											regAL.armyRes.setEnemyTank();
									}
								}
							}
							if(DialogBox.this.carr != null) {
								for(int i=0; i<tdspp.util.Constants.DIALOG_TEXTBOXES_updData.length; i++) {
									if(i == 0)
										regAL.armyRes.stateAmmunition = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 1)
										regAL.armyRes.stateFood = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 2)
										regAL.armyRes.statePopulation = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 3)
										regAL.weatherInfo.temperature = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 4)
										regAL.weatherInfo.humidity = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 5)
										regAL.weatherInfo.windSpeed = new Float(((TextField)carr[i]).getText()).floatValue();
									if(i == 6)
										regAL.weatherInfo.sunlight = (int)new Float(((TextField)carr[i]).getText()).floatValue();
								}
								for(int i=tdspp.util.Constants.DIALOG_TEXTBOXES_updData.length;i<carr.length; i++) {
									if(i == tdspp.util.Constants.DIALOG_TEXTBOXES_updData.length)
										regAL.terrainDesc.type = tdspp.util.Constants.DIALOG_TERRAINTYPE_VALS[((java.awt.List)carr[i]).getSelectedIndex()];
								}
							}
							dispose();
							System.out.println("regAl" + regAL);
							Prism.core.Event r = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_SETATTR);
							r.eventType=PrismConstants.REQUEST;
							r.addParameter(tdspp.util.MessageConstants.PARAM_NAME_SETATTR_REGION,regAL);
							frm.tp[selectedSubMapId].parent.parent.send(r);
							frm.tp[selectedSubMapId].updateCell(regAL);
						}
						});
					add(b);
					setSize(tdspp.util.Constants.MAINFRAME_WIDTH*7/8,tdspp.util.Constants.MAINFRAME_HEIGHT*7/8);
				}
				else if (type == HQConstants.DIALOG_BOXTYPE_MOVE) {
					int noOfChks = HQConstants.DIALOG_CHECKBOXES_updChk.length + HQConstants.DIALOG_CHECKBOXES_unUpdChk.length;
					arr = new Checkbox[noOfChks];
					arrNum = 0;
					for(int i=0;i<HQConstants.DIALOG_CHECKBOXES_updChk.length;i++) {

						if(i == 0 && reg.armyRes.isInfantry()) {
							arr[arrNum] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_updChk[i]);
							//arr[arrNum].setState(true);
							arrNum++;
						}
						if(i == 1 && reg.armyRes.isTank()) {
							arr[arrNum] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_updChk[i]);
							//arr[arrNum].setState(true);
							arrNum++;
						}
						//System.out.println("Arr Num 1 >> " + arrNum);
						if(arrNum > 0)
							add(arr[arrNum-1]);
					}
/*
					System.out.println("HQConstants.DIALOG_CHECKBOXES_unUpdChk.length" + HQConstants.DIALOG_CHECKBOXES_unUpdChk.length);
					System.out.println("reg.armyRes.isMineField() " + reg.armyRes.isMineField() );
					System.out.println("reg.armyRes.isEnemyInfantry()" + reg.armyRes.isEnemyInfantry());
					System.out.println("reg.armyRes.isEnemyTank() " + reg.armyRes.isEnemyTank() );
*/
					for(int i=0;i<HQConstants.DIALOG_CHECKBOXES_unUpdChk.length;i++) {
						if(i == 0 && reg.armyRes.isMineField()) {
							arr[arrNum] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_unUpdChk[i]);
							//arr[arrNum].setState(true);
							arrNum++;
						}

						if(i == 1 && reg.armyRes.isEnemyInfantry()) {
							arr[arrNum] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_unUpdChk[i]);
							//arr[arrNum].setState(true);
							arrNum++;
						}
						if(i == 2 && reg.armyRes.isEnemyTank()) {
							arr[arrNum] = new Checkbox(HQConstants.DIALOG_CHECKBOXES_unUpdChk[i]);
							//arr[arrNum].setState(true);
							arrNum++;
						}
						if(arrNum > 0)
							add(arr[arrNum-1]);
					}

					items = arrNum + 1;
					setLayout(new GridLayout(items,1));
					Button b = new Button("Move To .....");

					b.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent ev) {
							//MainFrame frm = BottomPanel.this.frm;
							//if(frm.cellX == -1) return;
							Region regAL = BottomPanel.this.parent.regions[selectedSubMapId][frm.cellX * HQConstants.REPOSITORY_CELLS_PER_SUBMAP + frm.cellY];
							int addCounter = 0, removeCounter = 0;
							String addResourceNames[], removeResourceNames[];
							addResourceNames = new String[5];
							removeResourceNames = new String[5];
							int counters[];

							counters = readDialogBox(regAL,addResourceNames, addCounter,removeResourceNames, removeCounter);
							addCounter = counters[0];
							removeCounter = counters[1];
							BottomPanel.this.parent.moveFlag[selectedSubMapId] = true;



							if(addCounter > 0) {
								RenderingAgentImp ra = BottomPanel.this.frm.parent;
								Prism.core.Event req = new Prism.core.Event(tdspp.util.MessageConstants.MESSAGE_NAME_MOVERESOURCES);
								req.eventType=PrismConstants.REQUEST;
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_OPERATIONFLAG, new Integer(2));
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_FROMREGION,regAL);
								req.addParameter(tdspp.util.MessageConstants.PARAM_NAME_MOVERESOURCES_RESOURCENAMES,addResourceNames);
								ra.saveRequest(req,selectedSubMapId);
							}


							dispose();
							//frm.tp[subMapId].updateCell(regAL);
						}
					});
					b.setSize(HQConstants.MAINFRAME_WIDTH/2,HQConstants.MAINFRAME_HEIGHT/16);
					add(b);
					setSize(HQConstants.MAINFRAME_WIDTH/4,HQConstants.MAINFRAME_HEIGHT*items/8);
				}
				else {
					setSize(tdspp.util.Constants.MAINFRAME_WIDTH*5/8,tdspp.util.Constants.MAINFRAME_HEIGHT*5/8);
				}
			}

			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dispose();
				}
			});

		}

		private int[] readDialogBox(Region regAL, String addResourceNames[], int addCounter,
							String removeResourceNames [], int removeCounter) {
			String name;

			if(arr != null) {
				for(int i = 0; i < arrNum; i++) {
					//System.out.println("addCounter = " + addCounter);
					if(arr[i].getState()) {
						//System.out.println("Label " + i + " " + arr[i].getLabel());
						name = new String(arr[i].getLabel());
						addResourceNames[addCounter] = name;
						addCounter++;
						//System.out.println("increased counter.......");
						if(name.equalsIgnoreCase(HQConstants.DIALOG_CHECKBOXES_updChk[0])) {
							if(!regAL.armyRes.isInfantry()) {
								regAL.armyRes.setStateAmmunition((float)1.0);
								regAL.armyRes.setStateFood((float)1.0);
								regAL.armyRes.setStatePopulation((float)1.0);
							}
							regAL.armyRes.setInfantry();
						}
						if(name.equalsIgnoreCase(HQConstants.DIALOG_CHECKBOXES_updChk[1])) {
							if(!regAL.armyRes.isTank()) {
								regAL.armyRes.setStateAmmunition((float)1.0);
								regAL.armyRes.setStateFood((float)1.0);
								regAL.armyRes.setStatePopulation((float)1.0);
							}
							regAL.armyRes.setTank();
						}
						if(name.equalsIgnoreCase(HQConstants.DIALOG_CHECKBOXES_unUpdChk[0]))
							regAL.armyRes.setMineField();
						if(name.equalsIgnoreCase(HQConstants.DIALOG_CHECKBOXES_unUpdChk[1]))
							regAL.armyRes.setEnemyInfantry();
						if(name.equalsIgnoreCase(HQConstants.DIALOG_CHECKBOXES_unUpdChk[2]))
							regAL.armyRes.setEnemyTank();
						//System.out.println("checked states.......");
					}
					else {
						removeResourceNames[removeCounter] = arr[i].getLabel();
						removeCounter++;
					}
				}
			}
			//System.out.println("add >> " + addCounter);
			//System.out.println("delete >> " + removeCounter);
			int counters[] = new int[2];
			counters[0] =  addCounter;
			counters[1] =  removeCounter;
			return counters;
		}
	}

	private boolean hasResources(Region reg) {
		return (reg.armyRes.isInfantry() || reg.armyRes.isTank() || reg.armyRes.isMineField() ||
			reg.armyRes.isEnemyInfantry() || reg.armyRes.isEnemyTank());
	}
}
